from google.protobuf import duration_pb2 as _duration_pb2
from tecton_proto.args import data_source__client_pb2 as _data_source__client_pb2
from tecton_proto.args import data_source_config__client_pb2 as _data_source_config__client_pb2
from tecton_proto.args import diff_options__client_pb2 as _diff_options__client_pb2
from tecton_proto.args import user_defined_function__client_pb2 as _user_defined_function__client_pb2
from tecton_proto.common import secret__client_pb2 as _secret__client_pb2
from tecton_proto.common import spark_schema__client_pb2 as _spark_schema__client_pb2
from tecton_proto.data import hive_metastore__client_pb2 as _hive_metastore__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor
FILE_DATA_SOURCE_FORMAT_CSV: FileDataSourceFormat
FILE_DATA_SOURCE_FORMAT_JSON: FileDataSourceFormat
FILE_DATA_SOURCE_FORMAT_PARQUET: FileDataSourceFormat

class BatchDataSource(_message.Message):
    __slots__ = ["batch_config", "bigquery", "data_delay", "date_partition_column", "datetime_partition_columns", "file", "hive_table", "pandas_data_source_function", "push_source_table", "pyarrow_data_source_function", "raw_batch_translator", "redshift_db", "secrets", "snowflake", "spark_data_source_function", "spark_schema", "timestamp_column_properties", "unity_table"]
    class SecretsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: _secret__client_pb2.SecretReference
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[_secret__client_pb2.SecretReference, _Mapping]] = ...) -> None: ...
    BATCH_CONFIG_FIELD_NUMBER: _ClassVar[int]
    BIGQUERY_FIELD_NUMBER: _ClassVar[int]
    DATA_DELAY_FIELD_NUMBER: _ClassVar[int]
    DATETIME_PARTITION_COLUMNS_FIELD_NUMBER: _ClassVar[int]
    DATE_PARTITION_COLUMN_FIELD_NUMBER: _ClassVar[int]
    FILE_FIELD_NUMBER: _ClassVar[int]
    HIVE_TABLE_FIELD_NUMBER: _ClassVar[int]
    PANDAS_DATA_SOURCE_FUNCTION_FIELD_NUMBER: _ClassVar[int]
    PUSH_SOURCE_TABLE_FIELD_NUMBER: _ClassVar[int]
    PYARROW_DATA_SOURCE_FUNCTION_FIELD_NUMBER: _ClassVar[int]
    RAW_BATCH_TRANSLATOR_FIELD_NUMBER: _ClassVar[int]
    REDSHIFT_DB_FIELD_NUMBER: _ClassVar[int]
    SECRETS_FIELD_NUMBER: _ClassVar[int]
    SNOWFLAKE_FIELD_NUMBER: _ClassVar[int]
    SPARK_DATA_SOURCE_FUNCTION_FIELD_NUMBER: _ClassVar[int]
    SPARK_SCHEMA_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_COLUMN_PROPERTIES_FIELD_NUMBER: _ClassVar[int]
    UNITY_TABLE_FIELD_NUMBER: _ClassVar[int]
    batch_config: _data_source_config__client_pb2.BatchConfig
    bigquery: BigqueryDataSource
    data_delay: _duration_pb2.Duration
    date_partition_column: str
    datetime_partition_columns: _containers.RepeatedCompositeFieldContainer[DatetimePartitionColumn]
    file: FileDataSource
    hive_table: _hive_metastore__client_pb2.HiveTableDataSource
    pandas_data_source_function: PandasBatchDataSourceFunction
    push_source_table: PushDataSource
    pyarrow_data_source_function: PyArrowBatchDataSourceFunction
    raw_batch_translator: _user_defined_function__client_pb2.UserDefinedFunction
    redshift_db: RedshiftDataSource
    secrets: _containers.MessageMap[str, _secret__client_pb2.SecretReference]
    snowflake: SnowflakeDataSource
    spark_data_source_function: SparkBatchDataSourceFunction
    spark_schema: _spark_schema__client_pb2.SparkSchema
    timestamp_column_properties: TimestampColumnProperties
    unity_table: UnityTableDataSource
    def __init__(self, hive_table: _Optional[_Union[_hive_metastore__client_pb2.HiveTableDataSource, _Mapping]] = ..., file: _Optional[_Union[FileDataSource, _Mapping]] = ..., redshift_db: _Optional[_Union[RedshiftDataSource, _Mapping]] = ..., snowflake: _Optional[_Union[SnowflakeDataSource, _Mapping]] = ..., spark_data_source_function: _Optional[_Union[SparkBatchDataSourceFunction, _Mapping]] = ..., unity_table: _Optional[_Union[UnityTableDataSource, _Mapping]] = ..., push_source_table: _Optional[_Union[PushDataSource, _Mapping]] = ..., pandas_data_source_function: _Optional[_Union[PandasBatchDataSourceFunction, _Mapping]] = ..., bigquery: _Optional[_Union[BigqueryDataSource, _Mapping]] = ..., pyarrow_data_source_function: _Optional[_Union[PyArrowBatchDataSourceFunction, _Mapping]] = ..., spark_schema: _Optional[_Union[_spark_schema__client_pb2.SparkSchema, _Mapping]] = ..., timestamp_column_properties: _Optional[_Union[TimestampColumnProperties, _Mapping]] = ..., batch_config: _Optional[_Union[_data_source_config__client_pb2.BatchConfig, _Mapping]] = ..., date_partition_column: _Optional[str] = ..., datetime_partition_columns: _Optional[_Iterable[_Union[DatetimePartitionColumn, _Mapping]]] = ..., raw_batch_translator: _Optional[_Union[_user_defined_function__client_pb2.UserDefinedFunction, _Mapping]] = ..., data_delay: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., secrets: _Optional[_Mapping[str, _secret__client_pb2.SecretReference]] = ...) -> None: ...

class BigqueryDataSource(_message.Message):
    __slots__ = ["credentials", "dataset", "location", "project_id", "query", "table"]
    CREDENTIALS_FIELD_NUMBER: _ClassVar[int]
    DATASET_FIELD_NUMBER: _ClassVar[int]
    LOCATION_FIELD_NUMBER: _ClassVar[int]
    PROJECT_ID_FIELD_NUMBER: _ClassVar[int]
    QUERY_FIELD_NUMBER: _ClassVar[int]
    TABLE_FIELD_NUMBER: _ClassVar[int]
    credentials: _secret__client_pb2.SecretReference
    dataset: str
    location: str
    project_id: str
    query: str
    table: str
    def __init__(self, project_id: _Optional[str] = ..., dataset: _Optional[str] = ..., location: _Optional[str] = ..., table: _Optional[str] = ..., query: _Optional[str] = ..., credentials: _Optional[_Union[_secret__client_pb2.SecretReference, _Mapping]] = ...) -> None: ...

class DatetimePartitionColumn(_message.Message):
    __slots__ = ["column_name", "format_string", "minimum_seconds"]
    COLUMN_NAME_FIELD_NUMBER: _ClassVar[int]
    FORMAT_STRING_FIELD_NUMBER: _ClassVar[int]
    MINIMUM_SECONDS_FIELD_NUMBER: _ClassVar[int]
    column_name: str
    format_string: str
    minimum_seconds: int
    def __init__(self, column_name: _Optional[str] = ..., format_string: _Optional[str] = ..., minimum_seconds: _Optional[int] = ...) -> None: ...

class FileDataSource(_message.Message):
    __slots__ = ["convert_to_glue_format", "format", "schema_override", "schema_uri", "uri"]
    CONVERT_TO_GLUE_FORMAT_FIELD_NUMBER: _ClassVar[int]
    FORMAT_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_OVERRIDE_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_URI_FIELD_NUMBER: _ClassVar[int]
    URI_FIELD_NUMBER: _ClassVar[int]
    convert_to_glue_format: bool
    format: FileDataSourceFormat
    schema_override: _spark_schema__client_pb2.SparkSchema
    schema_uri: str
    uri: str
    def __init__(self, uri: _Optional[str] = ..., format: _Optional[_Union[FileDataSourceFormat, str]] = ..., convert_to_glue_format: bool = ..., schema_uri: _Optional[str] = ..., schema_override: _Optional[_Union[_spark_schema__client_pb2.SparkSchema, _Mapping]] = ...) -> None: ...

class PandasBatchDataSourceFunction(_message.Message):
    __slots__ = ["function", "supports_time_filtering"]
    FUNCTION_FIELD_NUMBER: _ClassVar[int]
    SUPPORTS_TIME_FILTERING_FIELD_NUMBER: _ClassVar[int]
    function: _user_defined_function__client_pb2.UserDefinedFunction
    supports_time_filtering: bool
    def __init__(self, function: _Optional[_Union[_user_defined_function__client_pb2.UserDefinedFunction, _Mapping]] = ..., supports_time_filtering: bool = ...) -> None: ...

class PushDataSource(_message.Message):
    __slots__ = ["ingested_data_location"]
    INGESTED_DATA_LOCATION_FIELD_NUMBER: _ClassVar[int]
    ingested_data_location: str
    def __init__(self, ingested_data_location: _Optional[str] = ...) -> None: ...

class PyArrowBatchDataSourceFunction(_message.Message):
    __slots__ = ["function", "supports_time_filtering"]
    FUNCTION_FIELD_NUMBER: _ClassVar[int]
    SUPPORTS_TIME_FILTERING_FIELD_NUMBER: _ClassVar[int]
    function: _user_defined_function__client_pb2.UserDefinedFunction
    supports_time_filtering: bool
    def __init__(self, function: _Optional[_Union[_user_defined_function__client_pb2.UserDefinedFunction, _Mapping]] = ..., supports_time_filtering: bool = ...) -> None: ...

class RedshiftDataSource(_message.Message):
    __slots__ = ["cluster_id", "database", "endpoint", "query", "table", "temp_s3"]
    CLUSTER_ID_FIELD_NUMBER: _ClassVar[int]
    DATABASE_FIELD_NUMBER: _ClassVar[int]
    ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    QUERY_FIELD_NUMBER: _ClassVar[int]
    TABLE_FIELD_NUMBER: _ClassVar[int]
    TEMP_S3_FIELD_NUMBER: _ClassVar[int]
    cluster_id: str
    database: str
    endpoint: str
    query: str
    table: str
    temp_s3: str
    def __init__(self, endpoint: _Optional[str] = ..., cluster_id: _Optional[str] = ..., database: _Optional[str] = ..., table: _Optional[str] = ..., query: _Optional[str] = ..., temp_s3: _Optional[str] = ...) -> None: ...

class SnowflakeDataSource(_message.Message):
    __slots__ = ["snowflakeArgs"]
    SNOWFLAKEARGS_FIELD_NUMBER: _ClassVar[int]
    snowflakeArgs: _data_source__client_pb2.SnowflakeDataSourceArgs
    def __init__(self, snowflakeArgs: _Optional[_Union[_data_source__client_pb2.SnowflakeDataSourceArgs, _Mapping]] = ...) -> None: ...

class SparkBatchDataSourceFunction(_message.Message):
    __slots__ = ["function", "supports_time_filtering"]
    FUNCTION_FIELD_NUMBER: _ClassVar[int]
    SUPPORTS_TIME_FILTERING_FIELD_NUMBER: _ClassVar[int]
    function: _user_defined_function__client_pb2.UserDefinedFunction
    supports_time_filtering: bool
    def __init__(self, function: _Optional[_Union[_user_defined_function__client_pb2.UserDefinedFunction, _Mapping]] = ..., supports_time_filtering: bool = ...) -> None: ...

class TimestampColumnProperties(_message.Message):
    __slots__ = ["column_name", "format"]
    COLUMN_NAME_FIELD_NUMBER: _ClassVar[int]
    FORMAT_FIELD_NUMBER: _ClassVar[int]
    column_name: str
    format: str
    def __init__(self, column_name: _Optional[str] = ..., format: _Optional[str] = ...) -> None: ...

class UnityTableDataSource(_message.Message):
    __slots__ = ["access_mode", "catalog", "schema", "table"]
    ACCESS_MODE_FIELD_NUMBER: _ClassVar[int]
    CATALOG_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    TABLE_FIELD_NUMBER: _ClassVar[int]
    access_mode: _data_source__client_pb2.UnityCatalogAccessMode
    catalog: str
    schema: str
    table: str
    def __init__(self, catalog: _Optional[str] = ..., schema: _Optional[str] = ..., table: _Optional[str] = ..., access_mode: _Optional[_Union[_data_source__client_pb2.UnityCatalogAccessMode, str]] = ...) -> None: ...

class FileDataSourceFormat(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from tecton_proto.args import basic_info__client_pb2 as _basic_info__client_pb2
from tecton_proto.args import data_source__client_pb2 as _data_source__client_pb2
from tecton_proto.args import diff_options__client_pb2 as _diff_options__client_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class DiffTestArgs(_message.Message):
    __slots__ = ["info", "new_field", "old_field", "options", "passive_field", "recreate_suppressable_field", "recreate_suppressable_invalidate_checkpoints_field", "recreate_suppressable_invalidate_checkpoints_nested_field", "recreate_suppressable_nested_field", "recreate_suppressable_restart_stream_field", "renamed_message_field", "renamed_primitive_field", "test_args_id", "unannotated_needing_recreate"]
    INFO_FIELD_NUMBER: _ClassVar[int]
    NEW_FIELD_FIELD_NUMBER: _ClassVar[int]
    OLD_FIELD_FIELD_NUMBER: _ClassVar[int]
    OPTIONS_FIELD_NUMBER: _ClassVar[int]
    PASSIVE_FIELD_FIELD_NUMBER: _ClassVar[int]
    RECREATE_SUPPRESSABLE_FIELD_FIELD_NUMBER: _ClassVar[int]
    RECREATE_SUPPRESSABLE_INVALIDATE_CHECKPOINTS_FIELD_FIELD_NUMBER: _ClassVar[int]
    RECREATE_SUPPRESSABLE_INVALIDATE_CHECKPOINTS_NESTED_FIELD_FIELD_NUMBER: _ClassVar[int]
    RECREATE_SUPPRESSABLE_NESTED_FIELD_FIELD_NUMBER: _ClassVar[int]
    RECREATE_SUPPRESSABLE_RESTART_STREAM_FIELD_FIELD_NUMBER: _ClassVar[int]
    RENAMED_MESSAGE_FIELD_FIELD_NUMBER: _ClassVar[int]
    RENAMED_PRIMITIVE_FIELD_FIELD_NUMBER: _ClassVar[int]
    TEST_ARGS_ID_FIELD_NUMBER: _ClassVar[int]
    UNANNOTATED_NEEDING_RECREATE_FIELD_NUMBER: _ClassVar[int]
    info: _basic_info__client_pb2.BasicInfo
    new_field: DiffTestFoo
    old_field: DiffTestFoo
    options: _containers.RepeatedCompositeFieldContainer[_data_source__client_pb2.Option]
    passive_field: str
    recreate_suppressable_field: str
    recreate_suppressable_invalidate_checkpoints_field: str
    recreate_suppressable_invalidate_checkpoints_nested_field: DiffTestNestedInner
    recreate_suppressable_nested_field: DiffTestNestedInner
    recreate_suppressable_restart_stream_field: str
    renamed_message_field: DiffTestNestedInner
    renamed_primitive_field: str
    test_args_id: _id__client_pb2.Id
    unannotated_needing_recreate: str
    def __init__(self, test_args_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., info: _Optional[_Union[_basic_info__client_pb2.BasicInfo, _Mapping]] = ..., new_field: _Optional[_Union[DiffTestFoo, _Mapping]] = ..., old_field: _Optional[_Union[DiffTestFoo, _Mapping]] = ..., passive_field: _Optional[str] = ..., recreate_suppressable_field: _Optional[str] = ..., recreate_suppressable_nested_field: _Optional[_Union[DiffTestNestedInner, _Mapping]] = ..., recreate_suppressable_invalidate_checkpoints_field: _Optional[str] = ..., recreate_suppressable_invalidate_checkpoints_nested_field: _Optional[_Union[DiffTestNestedInner, _Mapping]] = ..., recreate_suppressable_restart_stream_field: _Optional[str] = ..., unannotated_needing_recreate: _Optional[str] = ..., options: _Optional[_Iterable[_Union[_data_source__client_pb2.Option, _Mapping]]] = ..., renamed_primitive_field: _Optional[str] = ..., renamed_message_field: _Optional[_Union[DiffTestNestedInner, _Mapping]] = ...) -> None: ...

class DiffTestFoo(_message.Message):
    __slots__ = ["field_a", "field_b"]
    FIELD_A_FIELD_NUMBER: _ClassVar[int]
    FIELD_B_FIELD_NUMBER: _ClassVar[int]
    field_a: str
    field_b: str
    def __init__(self, field_a: _Optional[str] = ..., field_b: _Optional[str] = ...) -> None: ...

class DiffTestNestedInner(_message.Message):
    __slots__ = ["inplace_field", "recreate_suppressable_field", "recreate_suppressable_restart_stream_field", "unannotated_field"]
    INPLACE_FIELD_FIELD_NUMBER: _ClassVar[int]
    RECREATE_SUPPRESSABLE_FIELD_FIELD_NUMBER: _ClassVar[int]
    RECREATE_SUPPRESSABLE_RESTART_STREAM_FIELD_FIELD_NUMBER: _ClassVar[int]
    UNANNOTATED_FIELD_FIELD_NUMBER: _ClassVar[int]
    inplace_field: str
    recreate_suppressable_field: str
    recreate_suppressable_restart_stream_field: str
    unannotated_field: str
    def __init__(self, inplace_field: _Optional[str] = ..., recreate_suppressable_field: _Optional[str] = ..., recreate_suppressable_restart_stream_field: _Optional[str] = ..., unannotated_field: _Optional[str] = ...) -> None: ...

class DiffTestNestedOuter(_message.Message):
    __slots__ = ["args"]
    ARGS_FIELD_NUMBER: _ClassVar[int]
    args: DiffTestArgs
    def __init__(self, args: _Optional[_Union[DiffTestArgs, _Mapping]] = ...) -> None: ...



from google.protobuf import duration_pb2 as _duration_pb2
from tecton_proto.amplitude import client_logging__client_pb2 as _client_logging__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class AmplitudeEvent(_message.Message):
    __slots__ = ["device_id", "event_properties", "event_type", "os_name", "os_version", "platform", "session_id", "timestamp", "user_id"]
    DEVICE_ID_FIELD_NUMBER: _ClassVar[int]
    EVENT_PROPERTIES_FIELD_NUMBER: _ClassVar[int]
    EVENT_TYPE_FIELD_NUMBER: _ClassVar[int]
    OS_NAME_FIELD_NUMBER: _ClassVar[int]
    OS_VERSION_FIELD_NUMBER: _ClassVar[int]
    PLATFORM_FIELD_NUMBER: _ClassVar[int]
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    USER_ID_FIELD_NUMBER: _ClassVar[int]
    device_id: str
    event_properties: AmplitudeEventProperties
    event_type: str
    os_name: str
    os_version: str
    platform: str
    session_id: int
    timestamp: int
    user_id: str
    def __init__(self, user_id: _Optional[str] = ..., device_id: _Optional[str] = ..., event_type: _Optional[str] = ..., platform: _Optional[str] = ..., session_id: _Optional[int] = ..., timestamp: _Optional[int] = ..., os_name: _Optional[str] = ..., os_version: _Optional[str] = ..., event_properties: _Optional[_Union[AmplitudeEventProperties, _Mapping]] = ...) -> None: ...

class AmplitudeEventProperties(_message.Message):
    __slots__ = ["caller_identity", "cluster_name", "error_message", "execution_time", "json_out", "num_fcos_changed", "num_total_fcos", "num_v3_fcos", "num_v5_fcos", "num_warnings", "params", "python_version", "sdk_method_invocation", "sdk_version", "status", "success", "suppress_recreates", "workspace"]
    class ParamsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    CALLER_IDENTITY_FIELD_NUMBER: _ClassVar[int]
    CLUSTER_NAME_FIELD_NUMBER: _ClassVar[int]
    ERROR_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    EXECUTION_TIME_FIELD_NUMBER: _ClassVar[int]
    JSON_OUT_FIELD_NUMBER: _ClassVar[int]
    NUM_FCOS_CHANGED_FIELD_NUMBER: _ClassVar[int]
    NUM_TOTAL_FCOS_FIELD_NUMBER: _ClassVar[int]
    NUM_V3_FCOS_FIELD_NUMBER: _ClassVar[int]
    NUM_V5_FCOS_FIELD_NUMBER: _ClassVar[int]
    NUM_WARNINGS_FIELD_NUMBER: _ClassVar[int]
    PARAMS_FIELD_NUMBER: _ClassVar[int]
    PYTHON_VERSION_FIELD_NUMBER: _ClassVar[int]
    SDK_METHOD_INVOCATION_FIELD_NUMBER: _ClassVar[int]
    SDK_VERSION_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    SUPPRESS_RECREATES_FIELD_NUMBER: _ClassVar[int]
    WORKSPACE_FIELD_NUMBER: _ClassVar[int]
    caller_identity: CallerIdentity
    cluster_name: str
    error_message: str
    execution_time: _duration_pb2.Duration
    json_out: bool
    num_fcos_changed: int
    num_total_fcos: int
    num_v3_fcos: int
    num_v5_fcos: int
    num_warnings: int
    params: _containers.ScalarMap[str, str]
    python_version: str
    sdk_method_invocation: _client_logging__client_pb2.SDKMethodInvocation
    sdk_version: str
    status: str
    success: bool
    suppress_recreates: bool
    workspace: str
    def __init__(self, cluster_name: _Optional[str] = ..., workspace: _Optional[str] = ..., sdk_version: _Optional[str] = ..., python_version: _Optional[str] = ..., execution_time: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., num_total_fcos: _Optional[int] = ..., num_fcos_changed: _Optional[int] = ..., num_v3_fcos: _Optional[int] = ..., num_v5_fcos: _Optional[int] = ..., suppress_recreates: bool = ..., json_out: bool = ..., success: bool = ..., error_message: _Optional[str] = ..., num_warnings: _Optional[int] = ..., params: _Optional[_Mapping[str, str]] = ..., sdk_method_invocation: _Optional[_Union[_client_logging__client_pb2.SDKMethodInvocation, _Mapping]] = ..., status: _Optional[str] = ..., caller_identity: _Optional[_Union[CallerIdentity, _Mapping]] = ...) -> None: ...

class CallerIdentity(_message.Message):
    __slots__ = ["id", "identity_type", "name"]
    IDENTITY_TYPE_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    id: str
    identity_type: str
    name: str
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ..., identity_type: _Optional[str] = ...) -> None: ...

class UploadRequest(_message.Message):
    __slots__ = ["api_key", "events"]
    API_KEY_FIELD_NUMBER: _ClassVar[int]
    EVENTS_FIELD_NUMBER: _ClassVar[int]
    api_key: str
    events: _containers.RepeatedCompositeFieldContainer[AmplitudeEvent]
    def __init__(self, api_key: _Optional[str] = ..., events: _Optional[_Iterable[_Union[AmplitudeEvent, _Mapping]]] = ...) -> None: ...

class UploadResponse(_message.Message):
    __slots__ = ["code", "error", "events_ingested", "missing_field", "payload_size_bytes", "server_upload_time"]
    CODE_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    EVENTS_INGESTED_FIELD_NUMBER: _ClassVar[int]
    MISSING_FIELD_FIELD_NUMBER: _ClassVar[int]
    PAYLOAD_SIZE_BYTES_FIELD_NUMBER: _ClassVar[int]
    SERVER_UPLOAD_TIME_FIELD_NUMBER: _ClassVar[int]
    code: int
    error: str
    events_ingested: int
    missing_field: str
    payload_size_bytes: int
    server_upload_time: int
    def __init__(self, code: _Optional[int] = ..., events_ingested: _Optional[int] = ..., payload_size_bytes: _Optional[int] = ..., server_upload_time: _Optional[int] = ..., error: _Optional[str] = ..., missing_field: _Optional[str] = ...) -> None: ...



from tecton_proto.auth import principal__client_pb2 as _principal__client_pb2
from tecton_proto.auth import service__client_pb2 as _service__client_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from tecton_proto.common import schema__client_pb2 as _schema__client_pb2
from tecton_proto.modelartifactservice import model_artifact_data__client_pb2 as _model_artifact_data__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class CompleteModelArtifactUploadRequest(_message.Message):
    __slots__ = ["model_artifact_id", "part_etags", "upload_id"]
    class PartEtagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: int
        value: str
        def __init__(self, key: _Optional[int] = ..., value: _Optional[str] = ...) -> None: ...
    MODEL_ARTIFACT_ID_FIELD_NUMBER: _ClassVar[int]
    PART_ETAGS_FIELD_NUMBER: _ClassVar[int]
    UPLOAD_ID_FIELD_NUMBER: _ClassVar[int]
    model_artifact_id: _id__client_pb2.Id
    part_etags: _containers.ScalarMap[int, str]
    upload_id: str
    def __init__(self, model_artifact_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., upload_id: _Optional[str] = ..., part_etags: _Optional[_Mapping[int, str]] = ...) -> None: ...

class CompleteModelArtifactUploadResponse(_message.Message):
    __slots__ = ["key"]
    KEY_FIELD_NUMBER: _ClassVar[int]
    key: str
    def __init__(self, key: _Optional[str] = ...) -> None: ...

class CreateModelArtifactRequest(_message.Message):
    __slots__ = ["artifact_files", "description", "environments", "file_hashes", "input_schema", "model_config_file_path", "model_file_path", "name", "output_schema", "tags", "type"]
    class FileHashesEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    ARTIFACT_FILES_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENTS_FIELD_NUMBER: _ClassVar[int]
    FILE_HASHES_FIELD_NUMBER: _ClassVar[int]
    INPUT_SCHEMA_FIELD_NUMBER: _ClassVar[int]
    MODEL_CONFIG_FILE_PATH_FIELD_NUMBER: _ClassVar[int]
    MODEL_FILE_PATH_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    OUTPUT_SCHEMA_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    artifact_files: _containers.RepeatedScalarFieldContainer[str]
    description: str
    environments: _containers.RepeatedScalarFieldContainer[str]
    file_hashes: _containers.ScalarMap[str, str]
    input_schema: _schema__client_pb2.Schema
    model_config_file_path: str
    model_file_path: str
    name: str
    output_schema: _schema__client_pb2.Schema
    tags: _containers.ScalarMap[str, str]
    type: _model_artifact_data__client_pb2.ModelType
    def __init__(self, name: _Optional[str] = ..., type: _Optional[_Union[_model_artifact_data__client_pb2.ModelType, str]] = ..., description: _Optional[str] = ..., file_hashes: _Optional[_Mapping[str, str]] = ..., input_schema: _Optional[_Union[_schema__client_pb2.Schema, _Mapping]] = ..., output_schema: _Optional[_Union[_schema__client_pb2.Schema, _Mapping]] = ..., tags: _Optional[_Mapping[str, str]] = ..., model_file_path: _Optional[str] = ..., model_config_file_path: _Optional[str] = ..., artifact_files: _Optional[_Iterable[str]] = ..., environments: _Optional[_Iterable[str]] = ...) -> None: ...

class CreateModelArtifactResponse(_message.Message):
    __slots__ = ["model_artifact_info"]
    MODEL_ARTIFACT_INFO_FIELD_NUMBER: _ClassVar[int]
    model_artifact_info: _model_artifact_data__client_pb2.ModelArtifactInfo
    def __init__(self, model_artifact_info: _Optional[_Union[_model_artifact_data__client_pb2.ModelArtifactInfo, _Mapping]] = ...) -> None: ...

class DeleteModelArtifactRequest(_message.Message):
    __slots__ = ["id", "name"]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    id: _id__client_pb2.Id
    name: str
    def __init__(self, id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., name: _Optional[str] = ...) -> None: ...

class DeleteModelArtifactResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class DescribeModelArtifactRequest(_message.Message):
    __slots__ = ["id", "name"]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    id: _id__client_pb2.Id
    name: str
    def __init__(self, id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., name: _Optional[str] = ...) -> None: ...

class DescribeModelArtifactResponse(_message.Message):
    __slots__ = ["model_artifact_info"]
    MODEL_ARTIFACT_INFO_FIELD_NUMBER: _ClassVar[int]
    model_artifact_info: _model_artifact_data__client_pb2.ModelArtifactInfo
    def __init__(self, model_artifact_info: _Optional[_Union[_model_artifact_data__client_pb2.ModelArtifactInfo, _Mapping]] = ...) -> None: ...

class FetchModelArtifactRequest(_message.Message):
    __slots__ = ["id", "name"]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    id: _id__client_pb2.Id
    name: str
    def __init__(self, id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., name: _Optional[str] = ...) -> None: ...

class FetchModelArtifactResponse(_message.Message):
    __slots__ = ["model_artifact_download_url", "model_config_download_url"]
    MODEL_ARTIFACT_DOWNLOAD_URL_FIELD_NUMBER: _ClassVar[int]
    MODEL_CONFIG_DOWNLOAD_URL_FIELD_NUMBER: _ClassVar[int]
    model_artifact_download_url: str
    model_config_download_url: str
    def __init__(self, model_artifact_download_url: _Optional[str] = ..., model_config_download_url: _Optional[str] = ...) -> None: ...

class GetModelArtifactUploadUrlRequest(_message.Message):
    __slots__ = ["model_artifact_id"]
    MODEL_ARTIFACT_ID_FIELD_NUMBER: _ClassVar[int]
    model_artifact_id: _id__client_pb2.Id
    def __init__(self, model_artifact_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ...) -> None: ...

class GetModelArtifactUploadUrlResponse(_message.Message):
    __slots__ = ["model_artifact_id", "model_config_upload_url", "upload_id"]
    MODEL_ARTIFACT_ID_FIELD_NUMBER: _ClassVar[int]
    MODEL_CONFIG_UPLOAD_URL_FIELD_NUMBER: _ClassVar[int]
    UPLOAD_ID_FIELD_NUMBER: _ClassVar[int]
    model_artifact_id: _id__client_pb2.Id
    model_config_upload_url: str
    upload_id: str
    def __init__(self, model_artifact_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., upload_id: _Optional[str] = ..., model_config_upload_url: _Optional[str] = ...) -> None: ...

class GetTectonModelInfoRequest(_message.Message):
    __slots__ = ["name"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class GetTectonModelInfoResponse(_message.Message):
    __slots__ = ["model_info"]
    MODEL_INFO_FIELD_NUMBER: _ClassVar[int]
    model_info: _model_artifact_data__client_pb2.ModelInfo
    def __init__(self, model_info: _Optional[_Union[_model_artifact_data__client_pb2.ModelInfo, _Mapping]] = ...) -> None: ...

class ListModelArtifactsRequest(_message.Message):
    __slots__ = ["id", "name"]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    id: _id__client_pb2.Id
    name: str
    def __init__(self, id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., name: _Optional[str] = ...) -> None: ...

class ListModelArtifactsResponse(_message.Message):
    __slots__ = ["models"]
    MODELS_FIELD_NUMBER: _ClassVar[int]
    models: _containers.RepeatedCompositeFieldContainer[_model_artifact_data__client_pb2.ModelArtifactInfo]
    def __init__(self, models: _Optional[_Iterable[_Union[_model_artifact_data__client_pb2.ModelArtifactInfo, _Mapping]]] = ...) -> None: ...

class ListTectonModelsRequest(_message.Message):
    __slots__ = ["filter"]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    filter: str
    def __init__(self, filter: _Optional[str] = ...) -> None: ...

class ListTectonModelsResponse(_message.Message):
    __slots__ = ["models"]
    MODELS_FIELD_NUMBER: _ClassVar[int]
    models: _containers.RepeatedCompositeFieldContainer[_model_artifact_data__client_pb2.ModelInfo]
    def __init__(self, models: _Optional[_Iterable[_Union[_model_artifact_data__client_pb2.ModelInfo, _Mapping]]] = ...) -> None: ...

class UpdateTectonModelRequest(_message.Message):
    __slots__ = ["artifact_files", "description", "environments", "id", "input_schema", "model_config_file_path", "model_file_path", "name", "output_schema", "tags", "type"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    ARTIFACT_FILES_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENTS_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    INPUT_SCHEMA_FIELD_NUMBER: _ClassVar[int]
    MODEL_CONFIG_FILE_PATH_FIELD_NUMBER: _ClassVar[int]
    MODEL_FILE_PATH_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    OUTPUT_SCHEMA_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    artifact_files: _containers.RepeatedScalarFieldContainer[str]
    description: str
    environments: _containers.RepeatedScalarFieldContainer[str]
    id: _id__client_pb2.Id
    input_schema: _schema__client_pb2.Schema
    model_config_file_path: str
    model_file_path: str
    name: str
    output_schema: _schema__client_pb2.Schema
    tags: _containers.ScalarMap[str, str]
    type: _model_artifact_data__client_pb2.ModelType
    def __init__(self, name: _Optional[str] = ..., id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., type: _Optional[_Union[_model_artifact_data__client_pb2.ModelType, str]] = ..., description: _Optional[str] = ..., input_schema: _Optional[_Union[_schema__client_pb2.Schema, _Mapping]] = ..., output_schema: _Optional[_Union[_schema__client_pb2.Schema, _Mapping]] = ..., tags: _Optional[_Mapping[str, str]] = ..., model_file_path: _Optional[str] = ..., model_config_file_path: _Optional[str] = ..., artifact_files: _Optional[_Iterable[str]] = ..., environments: _Optional[_Iterable[str]] = ...) -> None: ...

class UpdateTectonModelResponse(_message.Message):
    __slots__ = ["model_config_upload_url"]
    MODEL_CONFIG_UPLOAD_URL_FIELD_NUMBER: _ClassVar[int]
    model_config_upload_url: str
    def __init__(self, model_config_upload_url: _Optional[str] = ...) -> None: ...

class UploadModelArtifactPartRequest(_message.Message):
    __slots__ = ["model_artifact_id", "parent_upload_id", "part_number"]
    MODEL_ARTIFACT_ID_FIELD_NUMBER: _ClassVar[int]
    PARENT_UPLOAD_ID_FIELD_NUMBER: _ClassVar[int]
    PART_NUMBER_FIELD_NUMBER: _ClassVar[int]
    model_artifact_id: _id__client_pb2.Id
    parent_upload_id: str
    part_number: int
    def __init__(self, model_artifact_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., parent_upload_id: _Optional[str] = ..., part_number: _Optional[int] = ...) -> None: ...

class UploadModelArtifactPartResponse(_message.Message):
    __slots__ = ["upload_url"]
    UPLOAD_URL_FIELD_NUMBER: _ClassVar[int]
    upload_url: str
    def __init__(self, upload_url: _Optional[str] = ...) -> None: ...



from typeguard import typechecked

from tecton_core.specs import utils
from tecton_proto.common import secret__client_pb2 as secret_pb2


@utils.frozen_strict
class SecretSpec:
    """Base class for secret spec."""

    scope: str
    key: str
    is_local: bool

    @classmethod
    @typechecked
    def from_proto(cls, proto: secret_pb2.SecretReference) -> "SecretSpec":
        return cls(
            scope=proto.scope,
            key=proto.key,
            is_local=proto.is_local,
        )



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

BATCH: DataSourceType
DESCRIPTOR: _descriptor.FileDescriptor
PUSH_NO_BATCH: DataSourceType
PUSH_WITH_BATCH: DataSourceType
STREAM_WITH_BATCH: DataSourceType
UNKNOWN: DataSourceType

class DataSourceType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from tecton_proto.args import diff_options__client_pb2 as _diff_options__client_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class UserDefinedFunction(_message.Message):
    __slots__ = ["body", "isolate_function_deserialization", "name"]
    BODY_FIELD_NUMBER: _ClassVar[int]
    ISOLATE_FUNCTION_DESERIALIZATION_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    body: str
    isolate_function_deserialization: bool
    name: str
    def __init__(self, name: _Optional[str] = ..., body: _Optional[str] = ..., isolate_function_deserialization: bool = ...) -> None: ...



from tecton_proto.args import entity__client_pb2 as _entity__client_pb2
from tecton_proto.args import feature_service__client_pb2 as _feature_service__client_pb2
from tecton_proto.args import feature_view__client_pb2 as _feature_view__client_pb2
from tecton_proto.args import resource_provider__client_pb2 as _resource_provider__client_pb2
from tecton_proto.args import server_group__client_pb2 as _server_group__client_pb2
from tecton_proto.args import transformation__client_pb2 as _transformation__client_pb2
from tecton_proto.args import virtual_data_source__client_pb2 as _virtual_data_source__client_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class FcoArgs(_message.Message):
    __slots__ = ["entity", "feature_service", "feature_view", "resource_provider", "server_group", "transformation", "virtual_data_source"]
    ENTITY_FIELD_NUMBER: _ClassVar[int]
    FEATURE_SERVICE_FIELD_NUMBER: _ClassVar[int]
    FEATURE_VIEW_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_PROVIDER_FIELD_NUMBER: _ClassVar[int]
    SERVER_GROUP_FIELD_NUMBER: _ClassVar[int]
    TRANSFORMATION_FIELD_NUMBER: _ClassVar[int]
    VIRTUAL_DATA_SOURCE_FIELD_NUMBER: _ClassVar[int]
    entity: _entity__client_pb2.EntityArgs
    feature_service: _feature_service__client_pb2.FeatureServiceArgs
    feature_view: _feature_view__client_pb2.FeatureViewArgs
    resource_provider: _resource_provider__client_pb2.ResourceProviderArgs
    server_group: _server_group__client_pb2.ServerGroupArgs
    transformation: _transformation__client_pb2.TransformationArgs
    virtual_data_source: _virtual_data_source__client_pb2.VirtualDataSourceArgs
    def __init__(self, virtual_data_source: _Optional[_Union[_virtual_data_source__client_pb2.VirtualDataSourceArgs, _Mapping]] = ..., entity: _Optional[_Union[_entity__client_pb2.EntityArgs, _Mapping]] = ..., feature_view: _Optional[_Union[_feature_view__client_pb2.FeatureViewArgs, _Mapping]] = ..., feature_service: _Optional[_Union[_feature_service__client_pb2.FeatureServiceArgs, _Mapping]] = ..., transformation: _Optional[_Union[_transformation__client_pb2.TransformationArgs, _Mapping]] = ..., server_group: _Optional[_Union[_server_group__client_pb2.ServerGroupArgs, _Mapping]] = ..., resource_provider: _Optional[_Union[_resource_provider__client_pb2.ResourceProviderArgs, _Mapping]] = ...) -> None: ...



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

BATCH_COMPUTE_MODE_RIFT: BatchComputeMode
BATCH_COMPUTE_MODE_SNOWFLAKE: BatchComputeMode
BATCH_COMPUTE_MODE_SPARK: BatchComputeMode
BATCH_COMPUTE_MODE_UNSPECIFIED: BatchComputeMode
DESCRIPTOR: _descriptor.FileDescriptor

class BatchComputeMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from tecton_proto.spark_api import jobs__client_pb2 as _jobs__client_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

BATCH: TaskType
BATCH_JOB: TaskTypeForDisplay
COMPACTION_JOB: TaskTypeForDisplay
DATASET_GENERATION: TaskType
DATASET_GENERATION_JOB: TaskTypeForDisplay
DELETION: TaskType
DELETION_JOB: TaskTypeForDisplay
DELTA_MAINTENANCE: TaskType
DELTA_MAINTENANCE_JOB: TaskTypeForDisplay
DESCRIPTOR: _descriptor.FileDescriptor
ENV_DATABRICKS_NOTEBOOK: SparkExecutionEnvironment
ENV_DATAPROC: SparkExecutionEnvironment
ENV_EMR: SparkExecutionEnvironment
ENV_UNSPECIFIED: SparkExecutionEnvironment
FEATURE_EXPORT: TaskType
FEATURE_EXPORT_JOB: TaskTypeForDisplay
INGEST: TaskType
INGEST_JOB: TaskTypeForDisplay
PLAN_INTEGRATION_TEST_BATCH: TaskType
PLAN_INTEGRATION_TEST_BATCH_JOB: TaskTypeForDisplay
PLAN_INTEGRATION_TEST_STREAM: TaskType
PLAN_INTEGRATION_TEST_STREAM_JOB: TaskTypeForDisplay
STREAMING: TaskType
STREAMING_JOB: TaskTypeForDisplay
UNKNOWN: TaskType
UNKNOWN_JOB: TaskTypeForDisplay

class JobRequestTemplates(_message.Message):
    __slots__ = ["databricks_template", "emr_template"]
    DATABRICKS_TEMPLATE_FIELD_NUMBER: _ClassVar[int]
    EMR_TEMPLATE_FIELD_NUMBER: _ClassVar[int]
    databricks_template: _jobs__client_pb2.StartJobRequest
    emr_template: _jobs__client_pb2.StartJobRequest
    def __init__(self, databricks_template: _Optional[_Union[_jobs__client_pb2.StartJobRequest, _Mapping]] = ..., emr_template: _Optional[_Union[_jobs__client_pb2.StartJobRequest, _Mapping]] = ...) -> None: ...

class SparkClusterEnvironment(_message.Message):
    __slots__ = ["job_request_templates", "merged_user_deployment_settings_version", "spark_cluster_environment_version"]
    JOB_REQUEST_TEMPLATES_FIELD_NUMBER: _ClassVar[int]
    MERGED_USER_DEPLOYMENT_SETTINGS_VERSION_FIELD_NUMBER: _ClassVar[int]
    SPARK_CLUSTER_ENVIRONMENT_VERSION_FIELD_NUMBER: _ClassVar[int]
    job_request_templates: JobRequestTemplates
    merged_user_deployment_settings_version: int
    spark_cluster_environment_version: int
    def __init__(self, spark_cluster_environment_version: _Optional[int] = ..., job_request_templates: _Optional[_Union[JobRequestTemplates, _Mapping]] = ..., merged_user_deployment_settings_version: _Optional[int] = ...) -> None: ...

class TaskType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class TaskTypeForDisplay(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class SparkExecutionEnvironment(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from tecton_proto.data import feature_view__client_pb2 as _feature_view__client_pb2
from tecton_proto.online_store_writer import config__client_pb2 as _config__client_pb2
from tecton_proto.snowflake import snowflake_credentials__client_pb2 as _snowflake_credentials__client_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor
MICROS: TimestampUnit
MILLIS: TimestampUnit
UNSPECIFIED: TimestampUnit

class DeletionRequest(_message.Message):
    __slots__ = ["online_join_keys_full_path", "online_join_keys_path"]
    ONLINE_JOIN_KEYS_FULL_PATH_FIELD_NUMBER: _ClassVar[int]
    ONLINE_JOIN_KEYS_PATH_FIELD_NUMBER: _ClassVar[int]
    online_join_keys_full_path: str
    online_join_keys_path: str
    def __init__(self, online_join_keys_path: _Optional[str] = ..., online_join_keys_full_path: _Optional[str] = ...) -> None: ...

class GCSStage(_message.Message):
    __slots__ = ["blob", "bucket"]
    BLOB_FIELD_NUMBER: _ClassVar[int]
    BUCKET_FIELD_NUMBER: _ClassVar[int]
    blob: str
    bucket: str
    def __init__(self, bucket: _Optional[str] = ..., blob: _Optional[str] = ...) -> None: ...

class LocalFileStage(_message.Message):
    __slots__ = ["location"]
    LOCATION_FIELD_NUMBER: _ClassVar[int]
    location: str
    def __init__(self, location: _Optional[str] = ...) -> None: ...

class ObjectCopyRequest(_message.Message):
    __slots__ = ["gcs_stage", "local_file_stage", "s3_stage", "skip_rows", "snowflake_internal_stage", "timestamp_units"]
    GCS_STAGE_FIELD_NUMBER: _ClassVar[int]
    LOCAL_FILE_STAGE_FIELD_NUMBER: _ClassVar[int]
    S3_STAGE_FIELD_NUMBER: _ClassVar[int]
    SKIP_ROWS_FIELD_NUMBER: _ClassVar[int]
    SNOWFLAKE_INTERNAL_STAGE_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_UNITS_FIELD_NUMBER: _ClassVar[int]
    gcs_stage: GCSStage
    local_file_stage: LocalFileStage
    s3_stage: S3Stage
    skip_rows: int
    snowflake_internal_stage: SnowflakeInternalStage
    timestamp_units: TimestampUnit
    def __init__(self, s3_stage: _Optional[_Union[S3Stage, _Mapping]] = ..., snowflake_internal_stage: _Optional[_Union[SnowflakeInternalStage, _Mapping]] = ..., local_file_stage: _Optional[_Union[LocalFileStage, _Mapping]] = ..., gcs_stage: _Optional[_Union[GCSStage, _Mapping]] = ..., skip_rows: _Optional[int] = ..., timestamp_units: _Optional[_Union[TimestampUnit, str]] = ...) -> None: ...

class OnlineStoreCopierRequest(_message.Message):
    __slots__ = ["attempt_index", "deletion_request", "enable_new_consumption_metrics", "feature_view", "materialization_task_attempt_id", "object_copy_request", "online_store_metadata_configuration", "online_store_writer_configuration", "sqs_url", "status_update_request", "task_key"]
    ATTEMPT_INDEX_FIELD_NUMBER: _ClassVar[int]
    DELETION_REQUEST_FIELD_NUMBER: _ClassVar[int]
    ENABLE_NEW_CONSUMPTION_METRICS_FIELD_NUMBER: _ClassVar[int]
    FEATURE_VIEW_FIELD_NUMBER: _ClassVar[int]
    MATERIALIZATION_TASK_ATTEMPT_ID_FIELD_NUMBER: _ClassVar[int]
    OBJECT_COPY_REQUEST_FIELD_NUMBER: _ClassVar[int]
    ONLINE_STORE_METADATA_CONFIGURATION_FIELD_NUMBER: _ClassVar[int]
    ONLINE_STORE_WRITER_CONFIGURATION_FIELD_NUMBER: _ClassVar[int]
    SQS_URL_FIELD_NUMBER: _ClassVar[int]
    STATUS_UPDATE_REQUEST_FIELD_NUMBER: _ClassVar[int]
    TASK_KEY_FIELD_NUMBER: _ClassVar[int]
    attempt_index: int
    deletion_request: DeletionRequest
    enable_new_consumption_metrics: bool
    feature_view: _feature_view__client_pb2.FeatureView
    materialization_task_attempt_id: _id__client_pb2.Id
    object_copy_request: ObjectCopyRequest
    online_store_metadata_configuration: _config__client_pb2.OnlineStoreMetadataConfiguration
    online_store_writer_configuration: _config__client_pb2.OnlineStoreWriterConfiguration
    sqs_url: str
    status_update_request: StatusUpdateRequest
    task_key: str
    def __init__(self, online_store_writer_configuration: _Optional[_Union[_config__client_pb2.OnlineStoreWriterConfiguration, _Mapping]] = ..., feature_view: _Optional[_Union[_feature_view__client_pb2.FeatureView, _Mapping]] = ..., materialization_task_attempt_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., task_key: _Optional[str] = ..., attempt_index: _Optional[int] = ..., sqs_url: _Optional[str] = ..., online_store_metadata_configuration: _Optional[_Union[_config__client_pb2.OnlineStoreMetadataConfiguration, _Mapping]] = ..., enable_new_consumption_metrics: bool = ..., object_copy_request: _Optional[_Union[ObjectCopyRequest, _Mapping]] = ..., status_update_request: _Optional[_Union[StatusUpdateRequest, _Mapping]] = ..., deletion_request: _Optional[_Union[DeletionRequest, _Mapping]] = ...) -> None: ...

class S3Stage(_message.Message):
    __slots__ = ["bucket", "key"]
    BUCKET_FIELD_NUMBER: _ClassVar[int]
    KEY_FIELD_NUMBER: _ClassVar[int]
    bucket: str
    key: str
    def __init__(self, bucket: _Optional[str] = ..., key: _Optional[str] = ...) -> None: ...

class SnowflakeInternalStage(_message.Message):
    __slots__ = ["credentials", "location", "snowflake_account_identifier"]
    CREDENTIALS_FIELD_NUMBER: _ClassVar[int]
    LOCATION_FIELD_NUMBER: _ClassVar[int]
    SNOWFLAKE_ACCOUNT_IDENTIFIER_FIELD_NUMBER: _ClassVar[int]
    credentials: _snowflake_credentials__client_pb2.SnowflakeCredentials
    location: str
    snowflake_account_identifier: str
    def __init__(self, snowflake_account_identifier: _Optional[str] = ..., credentials: _Optional[_Union[_snowflake_credentials__client_pb2.SnowflakeCredentials, _Mapping]] = ..., location: _Optional[str] = ...) -> None: ...

class StatusUpdateRequest(_message.Message):
    __slots__ = ["anchor_time", "materialized_raw_data_end_time"]
    ANCHOR_TIME_FIELD_NUMBER: _ClassVar[int]
    MATERIALIZED_RAW_DATA_END_TIME_FIELD_NUMBER: _ClassVar[int]
    anchor_time: _timestamp_pb2.Timestamp
    materialized_raw_data_end_time: _timestamp_pb2.Timestamp
    def __init__(self, anchor_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., materialized_raw_data_end_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class TimestampUnit(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from typing import List

import attrs
from pypika import Field
from pypika.terms import Term

from tecton_core.compute_mode import ComputeMode
from tecton_core.feature_definition_wrapper import FeatureDefinitionWrapper
from tecton_core.query.calculation_sql_builder import CalculationSqlBuilderFactory
from tecton_core.query.column_reference_resolver import ColumnReferenceResolver


@attrs.define(init=False)
class FeatureExtractionSqlBuilder:
    """A class that generates pypika query representations of Calculation and Attribute Features."""

    _column_reference_resolver: ColumnReferenceResolver
    _compute_mode: ComputeMode
    _use_namespace_feature_prefix: bool = True

    def __init__(
        self,
        events_df_timestamp_field: str,
        input_node_column_names: List[str],
        compute_mode: ComputeMode,
        use_namespace_feature_prefix: bool,
    ) -> None:
        self._column_reference_resolver = ColumnReferenceResolver(events_df_timestamp_field, input_node_column_names)
        self._compute_mode = compute_mode
        self._use_namespace_feature_prefix = use_namespace_feature_prefix

    def _extract_calculations(self, fdw: FeatureDefinitionWrapper, namespace: str) -> List[Term]:
        separator = fdw.namespace_separator
        calculation_sql_builder = CalculationSqlBuilderFactory.create_builder(
            fdw, self._column_reference_resolver, self._compute_mode
        )
        calculation_query_terms = []
        for calc in fdw.fv_spec.calculation_features:
            output_column = f"{namespace}{separator}{calc.name}" if self._use_namespace_feature_prefix else calc.name
            calculation_query_term = calculation_sql_builder.ast_node_to_query_term(calc.root).as_(output_column)
            calculation_query_terms.append(calculation_query_term)
        return calculation_query_terms

    def _extract_attributes(self, fdw: FeatureDefinitionWrapper, namespace: str) -> List[Term]:
        separator = fdw.namespace_separator
        attribute_query_terms = []
        for attribute in fdw.fv_spec.attribute_features:
            internal_column_name = self._column_reference_resolver.get_internal_column_name(attribute.name, fdw)
            output_column = (
                f"{namespace}{separator}{attribute.name}" if self._use_namespace_feature_prefix else attribute.name
            )
            attribute_query_term = Field(internal_column_name).as_(output_column)
            attribute_query_terms.append(attribute_query_term)
        return attribute_query_terms

    def extract_rtfv_features(self, fdw: FeatureDefinitionWrapper, namespace: str) -> List[Term]:
        feature_query_terms = []
        if fdw.fv_spec.attribute_features:
            feature_query_terms.extend(self._extract_attributes(fdw, namespace))
        if fdw.fv_spec.calculation_features:
            feature_query_terms.extend(self._extract_calculations(fdw, namespace))
        return feature_query_terms



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.common import aws_credentials__client_pb2 as _aws_credentials__client_pb2
from tecton_proto.common import schema__client_pb2 as _schema__client_pb2
from tecton_proto.offlinestore.delta import metadata__client_pb2 as _metadata__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class AddFile(_message.Message):
    __slots__ = ["partition_values", "stats", "tags", "uri"]
    class PartitionValuesEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    PARTITION_VALUES_FIELD_NUMBER: _ClassVar[int]
    STATS_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    URI_FIELD_NUMBER: _ClassVar[int]
    partition_values: _containers.ScalarMap[str, str]
    stats: str
    tags: _containers.ScalarMap[str, str]
    uri: str
    def __init__(self, uri: _Optional[str] = ..., partition_values: _Optional[_Mapping[str, str]] = ..., tags: _Optional[_Mapping[str, str]] = ..., stats: _Optional[str] = ...) -> None: ...

class CrossAccountRoleConfig(_message.Message):
    __slots__ = ["dynamo_cross_account_role", "s3_cross_account_role"]
    DYNAMO_CROSS_ACCOUNT_ROLE_FIELD_NUMBER: _ClassVar[int]
    S3_CROSS_ACCOUNT_ROLE_FIELD_NUMBER: _ClassVar[int]
    dynamo_cross_account_role: _aws_credentials__client_pb2.AwsIamRole
    s3_cross_account_role: _aws_credentials__client_pb2.AwsIamRole
    def __init__(self, s3_cross_account_role: _Optional[_Union[_aws_credentials__client_pb2.AwsIamRole, _Mapping]] = ..., dynamo_cross_account_role: _Optional[_Union[_aws_credentials__client_pb2.AwsIamRole, _Mapping]] = ...) -> None: ...

class Expression(_message.Message):
    __slots__ = ["binary", "column", "literal"]
    class Binary(_message.Message):
        __slots__ = ["left", "op", "right"]
        class Op(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
            __slots__ = []
        LEFT_FIELD_NUMBER: _ClassVar[int]
        OP_AND: Expression.Binary.Op
        OP_EQ: Expression.Binary.Op
        OP_FIELD_NUMBER: _ClassVar[int]
        OP_LE: Expression.Binary.Op
        OP_LT: Expression.Binary.Op
        OP_OR: Expression.Binary.Op
        OP_UNSPECIFIED: Expression.Binary.Op
        RIGHT_FIELD_NUMBER: _ClassVar[int]
        left: Expression
        op: Expression.Binary.Op
        right: Expression
        def __init__(self, op: _Optional[_Union[Expression.Binary.Op, str]] = ..., left: _Optional[_Union[Expression, _Mapping]] = ..., right: _Optional[_Union[Expression, _Mapping]] = ...) -> None: ...
    class Literal(_message.Message):
        __slots__ = ["bool", "int64", "str", "timestamp"]
        BOOL_FIELD_NUMBER: _ClassVar[int]
        INT64_FIELD_NUMBER: _ClassVar[int]
        STR_FIELD_NUMBER: _ClassVar[int]
        TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
        bool: bool
        int64: int
        str: str
        timestamp: _timestamp_pb2.Timestamp
        def __init__(self, str: _Optional[str] = ..., timestamp: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., int64: _Optional[int] = ..., bool: bool = ...) -> None: ...
    BINARY_FIELD_NUMBER: _ClassVar[int]
    COLUMN_FIELD_NUMBER: _ClassVar[int]
    LITERAL_FIELD_NUMBER: _ClassVar[int]
    binary: Expression.Binary
    column: _schema__client_pb2.Column
    literal: Expression.Literal
    def __init__(self, column: _Optional[_Union[_schema__client_pb2.Column, _Mapping]] = ..., literal: _Optional[_Union[Expression.Literal, _Mapping]] = ..., binary: _Optional[_Union[Expression.Binary, _Mapping]] = ...) -> None: ...

class GetPartitionsArgs(_message.Message):
    __slots__ = ["tags"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    TAGS_FIELD_NUMBER: _ClassVar[int]
    tags: _containers.ScalarMap[str, str]
    def __init__(self, tags: _Optional[_Mapping[str, str]] = ...) -> None: ...

class GetPartitionsResult(_message.Message):
    __slots__ = ["partitions"]
    class Partition(_message.Message):
        __slots__ = ["values"]
        class ValuesEntry(_message.Message):
            __slots__ = ["key", "value"]
            KEY_FIELD_NUMBER: _ClassVar[int]
            VALUE_FIELD_NUMBER: _ClassVar[int]
            key: str
            value: str
            def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
        VALUES_FIELD_NUMBER: _ClassVar[int]
        values: _containers.ScalarMap[str, str]
        def __init__(self, values: _Optional[_Mapping[str, str]] = ...) -> None: ...
    PARTITIONS_FIELD_NUMBER: _ClassVar[int]
    partitions: _containers.RepeatedCompositeFieldContainer[GetPartitionsResult.Partition]
    def __init__(self, partitions: _Optional[_Iterable[_Union[GetPartitionsResult.Partition, _Mapping]]] = ...) -> None: ...

class InitializeArgs(_message.Message):
    __slots__ = ["cross_account_role_configs", "description", "dynamodb_log_table_name", "dynamodb_log_table_region", "id", "kms_key_arn", "name", "partition_columns", "path", "schema"]
    CROSS_ACCOUNT_ROLE_CONFIGS_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    DYNAMODB_LOG_TABLE_NAME_FIELD_NUMBER: _ClassVar[int]
    DYNAMODB_LOG_TABLE_REGION_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    KMS_KEY_ARN_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    PARTITION_COLUMNS_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    cross_account_role_configs: CrossAccountRoleConfig
    description: str
    dynamodb_log_table_name: str
    dynamodb_log_table_region: str
    id: str
    kms_key_arn: str
    name: str
    partition_columns: _containers.RepeatedScalarFieldContainer[str]
    path: str
    schema: _schema__client_pb2.Schema
    def __init__(self, path: _Optional[str] = ..., id: _Optional[str] = ..., name: _Optional[str] = ..., description: _Optional[str] = ..., schema: _Optional[_Union[_schema__client_pb2.Schema, _Mapping]] = ..., partition_columns: _Optional[_Iterable[str]] = ..., dynamodb_log_table_name: _Optional[str] = ..., dynamodb_log_table_region: _Optional[str] = ..., cross_account_role_configs: _Optional[_Union[CrossAccountRoleConfig, _Mapping]] = ..., kms_key_arn: _Optional[str] = ...) -> None: ...

class ReadForUpdateArgs(_message.Message):
    __slots__ = ["read_predicate"]
    READ_PREDICATE_FIELD_NUMBER: _ClassVar[int]
    read_predicate: Expression
    def __init__(self, read_predicate: _Optional[_Union[Expression, _Mapping]] = ...) -> None: ...

class ReadForUpdateResult(_message.Message):
    __slots__ = ["uris"]
    URIS_FIELD_NUMBER: _ClassVar[int]
    uris: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, uris: _Optional[_Iterable[str]] = ...) -> None: ...

class UpdateArgs(_message.Message):
    __slots__ = ["add_files", "delete_uris", "user_metadata"]
    ADD_FILES_FIELD_NUMBER: _ClassVar[int]
    DELETE_URIS_FIELD_NUMBER: _ClassVar[int]
    USER_METADATA_FIELD_NUMBER: _ClassVar[int]
    add_files: _containers.RepeatedCompositeFieldContainer[AddFile]
    delete_uris: _containers.RepeatedScalarFieldContainer[str]
    user_metadata: _metadata__client_pb2.TectonDeltaMetadata
    def __init__(self, add_files: _Optional[_Iterable[_Union[AddFile, _Mapping]]] = ..., user_metadata: _Optional[_Union[_metadata__client_pb2.TectonDeltaMetadata, _Mapping]] = ..., delete_uris: _Optional[_Iterable[str]] = ...) -> None: ...

class UpdateResult(_message.Message):
    __slots__ = ["committed_version", "error_message", "error_type"]
    class ErrorType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    COMMITTED_VERSION_FIELD_NUMBER: _ClassVar[int]
    CONCURRENT_APPEND_ERROR: UpdateResult.ErrorType
    CONCURRENT_DELETE_DELETE_ERROR: UpdateResult.ErrorType
    CONCURRENT_DELETE_READ_ERROR: UpdateResult.ErrorType
    CONCURRENT_TRANSACTION_ERROR: UpdateResult.ErrorType
    ERROR_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    ERROR_TYPE_FIELD_NUMBER: _ClassVar[int]
    ERROR_UNKNOWN: UpdateResult.ErrorType
    ERROR_UNSPECIFIED: UpdateResult.ErrorType
    METADATA_CHANGED_ERROR: UpdateResult.ErrorType
    PROTOCOL_CHANGED_ERROR: UpdateResult.ErrorType
    committed_version: int
    error_message: str
    error_type: UpdateResult.ErrorType
    def __init__(self, committed_version: _Optional[int] = ..., error_type: _Optional[_Union[UpdateResult.ErrorType, str]] = ..., error_message: _Optional[str] = ...) -> None: ...



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class ErrorResponse(_message.Message):
    __slots__ = ["error_code", "message"]
    ERROR_CODE_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    error_code: str
    message: str
    def __init__(self, error_code: _Optional[str] = ..., message: _Optional[str] = ...) -> None: ...



from abc import ABC
from abc import abstractmethod
from typing import Dict
from typing import Set

import attrs
import pypika.functions as fn
from pypika import NULL
from pypika.enums import Arithmetic
from pypika.enums import Equality
from pypika.terms import ArithmeticExpression
from pypika.terms import BasicCriterion
from pypika.terms import Case
from pypika.terms import Field
from pypika.terms import LiteralValue
from pypika.terms import Not
from pypika.terms import Term
from pypika.terms import ValueWrapper

from tecton_core import data_types
from tecton_core.compute_mode import ComputeMode
from tecton_core.errors import TectonInternalError
from tecton_core.feature_definition_wrapper import FeatureDefinitionWrapper
from tecton_core.query.column_reference_resolver import ColumnReferenceResolver
from tecton_core.specs.calculation_node_spec import AbstractSyntaxTreeNodeSpec
from tecton_core.specs.calculation_node_spec import CaseStatementNodeSpec
from tecton_core.specs.calculation_node_spec import ColumnReferenceNodeSpec
from tecton_core.specs.calculation_node_spec import LiteralValueNodeSpec
from tecton_core.specs.calculation_node_spec import OperationNodeSpec
from tecton_proto.common import calculation_node__client_pb2 as calculation_node_pb2
from tecton_proto.common.calculation_node__client_pb2 import OperationType


COMPARISON_OPERATION_TYPE_TO_PYPIKA_EQUALITY: Dict[calculation_node_pb2.OperationType.ValueType, Equality] = {
    calculation_node_pb2.OperationType.EQUALS: Equality.eq,
    calculation_node_pb2.OperationType.NOT_EQUALS: Equality.ne,
    calculation_node_pb2.OperationType.GREATER_THAN: Equality.gt,
    calculation_node_pb2.OperationType.GREATER_THAN_EQUALS: Equality.gte,
    calculation_node_pb2.OperationType.LESS_THAN: Equality.lt,
    calculation_node_pb2.OperationType.LESS_THAN_EQUALS: Equality.lte,
}

DATE_PART_TO_INTERVAL_STRING: Dict[calculation_node_pb2.DatePart.ValueType, str] = {
    calculation_node_pb2.DatePart.DAY: "day",
    calculation_node_pb2.DatePart.MONTH: "month",
    calculation_node_pb2.DatePart.WEEK: "week",
    calculation_node_pb2.DatePart.YEAR: "year",
    calculation_node_pb2.DatePart.SECOND: "second",
    calculation_node_pb2.DatePart.HOUR: "hour",
    calculation_node_pb2.DatePart.MINUTE: "minute",
    calculation_node_pb2.DatePart.MILLENNIUM: "millennium",
    calculation_node_pb2.DatePart.CENTURY: "century",
    calculation_node_pb2.DatePart.DECADE: "decade",
    calculation_node_pb2.DatePart.QUARTER: "quarter",
    calculation_node_pb2.DatePart.MILLISECONDS: "milliseconds",
    calculation_node_pb2.DatePart.MICROSECONDS: "microseconds",
}

ARITHMETIC_OPERATION_TYPE_TO_PYPIKA_ARITHMETIC: Dict[calculation_node_pb2.OperationType.ValueType, Arithmetic] = {
    calculation_node_pb2.OperationType.ADDITION: Arithmetic.add,
    calculation_node_pb2.OperationType.SUBTRACTION: Arithmetic.sub,
    calculation_node_pb2.OperationType.MULTIPLICATION: Arithmetic.mul,
    calculation_node_pb2.OperationType.DIVISION: Arithmetic.div,
}

LOGICAL_OPERATOR_OPERATIONS: Set[calculation_node_pb2.OperationType.ValueType] = {
    calculation_node_pb2.OperationType.NOT,
    calculation_node_pb2.OperationType.AND,
    calculation_node_pb2.OperationType.OR,
}


def _get_cast_string_for_numeric_dtype(dtype: data_types.DataType) -> str:
    if isinstance(dtype, data_types.Int32Type):
        return "INTEGER"
    elif isinstance(dtype, data_types.Int64Type):
        return "BIGINT"
    elif isinstance(dtype, data_types.Float32Type):
        return "FLOAT"
    elif isinstance(dtype, data_types.Float64Type):
        return "DOUBLE"
    else:
        msg = f"Data type {dtype} not supported for cast."
        raise TectonInternalError(msg)


@attrs.define
class CalculationSqlBuilder(ABC):
    fdw: FeatureDefinitionWrapper
    column_reference_resolver: ColumnReferenceResolver

    def _operation_node_to_query_term(self, operation_node: OperationNodeSpec) -> Term:
        if operation_node.operation == calculation_node_pb2.OperationType.COALESCE:
            return self._build_coalesce_query(operation_node)
        elif operation_node.operation in COMPARISON_OPERATION_TYPE_TO_PYPIKA_EQUALITY:
            return self._build_comparison_query(operation_node)
        elif operation_node.operation == calculation_node_pb2.OperationType.DATE_DIFF:
            return self._build_date_diff_query(operation_node)
        elif operation_node.operation in ARITHMETIC_OPERATION_TYPE_TO_PYPIKA_ARITHMETIC:
            return self._build_arithmetic_query(operation_node)
        elif operation_node.operation in LOGICAL_OPERATOR_OPERATIONS:
            return self._build_logical_operator_query(operation_node)
        else:
            msg = f"In Calculation sql generation, calculation operation {OperationType.Name(operation_node.operation)} not supported."
            raise TectonInternalError(msg)

    def _build_logical_operator_query(self, operation_node: OperationNodeSpec) -> Term:
        if operation_node.operation == calculation_node_pb2.OperationType.NOT:
            return self._build_logical_not_query(operation_node)
        elif operation_node.operation == calculation_node_pb2.OperationType.AND:
            return self._build_logical_and_query(operation_node)
        elif operation_node.operation == calculation_node_pb2.OperationType.OR:
            return self._build_logical_or_query(operation_node)
        else:
            msg = "Unhandled logical operator."
            raise TectonInternalError(msg)

    def _build_logical_not_query(self, operation_node: OperationNodeSpec) -> Term:
        if len(operation_node.operands) != 1:
            msg = "Calculation NOT operator must have exactly 1 operand."
            raise TectonInternalError(msg)

        operand_sql = fn.Cast(self.ast_node_to_query_term(operation_node.operands[0]), "BOOLEAN")
        return Not(operand_sql)

    def _build_logical_and_query(self, operation_node: OperationNodeSpec) -> Term:
        if len(operation_node.operands) != 2:
            msg = "Calculation AND operator must have exactly 2 operands."
            raise TectonInternalError(msg)

        operand_sqls = [
            fn.Cast(self.ast_node_to_query_term(operand), "BOOLEAN").get_sql() for operand in operation_node.operands
        ]
        return LiteralValue(f"{operand_sqls[0]} AND {operand_sqls[1]}")

    def _build_logical_or_query(self, operation_node: OperationNodeSpec) -> Term:
        if len(operation_node.operands) != 2:
            msg = "Calculation OR operator must have exactly 2 operands."
            raise TectonInternalError(msg)

        operand_sqls = [
            fn.Cast(self.ast_node_to_query_term(operand), "BOOLEAN").get_sql() for operand in operation_node.operands
        ]
        return LiteralValue(f"{operand_sqls[0]} OR {operand_sqls[1]}")

    def _build_coalesce_query(self, operation_node: OperationNodeSpec) -> Term:
        if len(operation_node.operands) < 1:
            msg = "Calculation function Coalesce must have at least 1 operand."
            raise TectonInternalError(msg)
        operand_sqls = [self.ast_node_to_query_term(operand) for operand in operation_node.operands]
        return fn.Coalesce(*operand_sqls)

    def _build_arithmetic_query(self, operation_node: OperationNodeSpec) -> Term:
        if len(operation_node.operands) != 2:
            msg = "Arithmetic function must have exactly 2 operands."
            raise TectonInternalError(msg)
        left = self.ast_node_to_query_term(operation_node.operands[0])
        right = self.ast_node_to_query_term(operation_node.operands[1])

        arithmetic_term = ARITHMETIC_OPERATION_TYPE_TO_PYPIKA_ARITHMETIC[operation_node.operation]
        base_expression = ArithmeticExpression(arithmetic_term, left, right)
        if operation_node.operation == calculation_node_pb2.OperationType.DIVISION:
            # this cast is necessary because:
            # in duckdb, x/0 returns inf, -x/0 returns -inf as of 1.1.0 (https://duckdb.org/2024/09/09/announcing-duckdb-110.html)
            # in spark, x/0 returns null.
            # we want to match duckdb behavior, so return inf if dividing by 0.
            positive_inf_literal = LiteralValue("CAST('inf' AS DOUBLE)")
            negative__inf_literal = LiteralValue("CAST('-inf' AS DOUBLE)")
            nan_literal = LiteralValue("CAST('nan' AS DOUBLE)")

            base_expression = (
                Case()
                .when((left == 0) & (right == 0), nan_literal)  # 0/0, return NaN
                .when((left < 0) & (right == 0), negative__inf_literal)  # -x/0, return -inf
                .when((left > 0) & (right == 0), positive_inf_literal)  # +x/0, return inf
                .else_(base_expression)  # Default case, use base_expression
            )

        cast_type = _get_cast_string_for_numeric_dtype(dtype=operation_node.dtype)
        return fn.Cast(base_expression, cast_type)

    def _build_comparison_query(self, operation_node: OperationNodeSpec) -> Term:
        if len(operation_node.operands) != 2:
            msg = "Calculation function must have exactly 2 operands."
            raise TectonInternalError(msg)
        left = self.ast_node_to_query_term(operation_node.operands[0])
        right = self.ast_node_to_query_term(operation_node.operands[1])

        comparator_term = COMPARISON_OPERATION_TYPE_TO_PYPIKA_EQUALITY[operation_node.operation]
        return BasicCriterion(comparator_term, left, right)

    @abstractmethod
    def _build_date_diff_query(self, operation_node: OperationNodeSpec) -> Term:
        raise NotImplementedError

    @staticmethod
    def _literal_value_node_to_query_term(literal_value_node: LiteralValueNodeSpec) -> Term:
        if literal_value_node.dtype is None or literal_value_node.value is None:
            return NULL
        sql = ValueWrapper(literal_value_node.value)
        if isinstance(literal_value_node.dtype, data_types.Int64Type):
            sql = fn.Cast(sql, _get_cast_string_for_numeric_dtype(dtype=literal_value_node.dtype))
        elif isinstance(literal_value_node.dtype, data_types.Float64Type):
            sql = fn.Cast(sql, _get_cast_string_for_numeric_dtype(dtype=literal_value_node.dtype))
        return sql

    def _column_reference_node_to_query_term(self, column_reference_node: ColumnReferenceNodeSpec) -> Term:
        internal_column_name = self.column_reference_resolver.get_internal_column_name(
            column_reference_node.value, self.fdw
        )
        return Field(internal_column_name)

    def _case_statement_node_to_query_term(self, case_statement_node: CaseStatementNodeSpec) -> Term:
        base_expression = Case()
        for when_clause in case_statement_node.when_clauses:
            condition = fn.Cast(self.ast_node_to_query_term(when_clause.condition), "BOOLEAN")
            result = self.ast_node_to_query_term(when_clause.result)
            base_expression = base_expression.when(condition, result)
        if case_statement_node.else_clause is not None:
            result = self.ast_node_to_query_term(case_statement_node.else_clause.result)
            base_expression = base_expression.else_(result)
        return base_expression

    def ast_node_to_query_term(self, ast_node: AbstractSyntaxTreeNodeSpec) -> Term:
        if isinstance(ast_node, OperationNodeSpec):
            return self._operation_node_to_query_term(ast_node)
        elif isinstance(ast_node, LiteralValueNodeSpec):
            return self._literal_value_node_to_query_term(ast_node)
        elif isinstance(ast_node, ColumnReferenceNodeSpec):
            return self._column_reference_node_to_query_term(ast_node)
        elif isinstance(ast_node, CaseStatementNodeSpec):
            return self._case_statement_node_to_query_term(ast_node)
        else:
            msg = f"AST node type {ast_node.__class__.__name__} not recognized. Cannot extract calculation."
            raise TectonInternalError(msg)


@attrs.define
class DuckDBCalculationSqlBuilder(CalculationSqlBuilder):
    def _build_date_diff_query(self, operation_node: OperationNodeSpec) -> Term:
        if len(operation_node.operands) != 3:
            msg = "Calculation function date diff must have exactly 3 operands."
            raise TectonInternalError(msg)
        [date_part_operand, start_date_operand, end_date_operand] = operation_node.operands
        date_part_str = DATE_PART_TO_INTERVAL_STRING[date_part_operand.value]
        start_date_operand = self.ast_node_to_query_term(start_date_operand)
        end_date_operand = self.ast_node_to_query_term(end_date_operand)

        return fn.DateDiff(date_part_str, start_date_operand, end_date_operand)


@attrs.define
class SparkCalculationSqlBuilder(CalculationSqlBuilder):
    @staticmethod
    def _get_spark_date_diff_sql_str(
        date_part: calculation_node_pb2.DatePart.ValueType, start_date_sql: str, end_date_sql: str
    ) -> str:
        if date_part == calculation_node_pb2.DatePart.DAY:
            return f"FLOOR(DATEDIFF(DATE_TRUNC('day', {end_date_sql}), DATE_TRUNC('day', {start_date_sql})))"
        elif date_part == calculation_node_pb2.DatePart.MONTH:
            return f"FLOOR(MONTHS_BETWEEN(DATE_TRUNC('month', {end_date_sql}), DATE_TRUNC('month', {start_date_sql})))"
        elif date_part == calculation_node_pb2.DatePart.WEEK:
            return f"FLOOR(DATEDIFF({end_date_sql}, {start_date_sql})/7)"
        elif date_part == calculation_node_pb2.DatePart.YEAR:
            return f"YEAR({end_date_sql}) - YEAR({start_date_sql})"
        elif date_part == calculation_node_pb2.DatePart.SECOND:
            return f"UNIX_TIMESTAMP(DATE_TRUNC('second', {end_date_sql})) - UNIX_TIMESTAMP(DATE_TRUNC('second', {start_date_sql}))"
        elif date_part == calculation_node_pb2.DatePart.HOUR:
            return f"( UNIX_TIMESTAMP(DATE_TRUNC('hour', {end_date_sql})) - UNIX_TIMESTAMP(DATE_TRUNC('hour', {start_date_sql})) ) / 3600"
        elif date_part == calculation_node_pb2.DatePart.MINUTE:
            return f"( UNIX_TIMESTAMP(DATE_TRUNC('minute', {end_date_sql})) - UNIX_TIMESTAMP(DATE_TRUNC('minute', {start_date_sql})) ) / 60"
        elif date_part == calculation_node_pb2.DatePart.MILLENNIUM:
            return f"FLOOR(YEAR({end_date_sql}) / 1000) - FLOOR(YEAR({start_date_sql}) / 1000)"
        elif date_part == calculation_node_pb2.DatePart.CENTURY:
            return f"FLOOR(YEAR({end_date_sql}) / 100) - FLOOR(YEAR({start_date_sql}) / 100)"
        elif date_part == calculation_node_pb2.DatePart.DECADE:
            return f"FLOOR(YEAR({end_date_sql}) / 10) - FLOOR(YEAR({start_date_sql}) / 10)"
        elif date_part == calculation_node_pb2.DatePart.QUARTER:
            return f"4 * (YEAR({end_date_sql}) - YEAR({start_date_sql})) + FLOOR((MONTH({end_date_sql})-1 )/ 3) - FLOOR((MONTH({start_date_sql})-1 )/ 3)"
        elif date_part == calculation_node_pb2.DatePart.MILLISECONDS:
            return f"UNIX_MILLIS({end_date_sql}) - UNIX_MILLIS({start_date_sql})"
        elif date_part == calculation_node_pb2.DatePart.MICROSECONDS:
            return f"UNIX_MICROS({end_date_sql}) - UNIX_MICROS({start_date_sql})"
        else:
            msg = f"Date part {date_part} is not supported."
            raise TectonInternalError(msg)

    def _build_date_diff_query(self, operation_node: OperationNodeSpec) -> Term:
        if len(operation_node.operands) != 3:
            msg = "Calculation function date diff must have exactly 3 operands."
            raise TectonInternalError(msg)

        [date_part_operand, start_date_operand, end_date_operand] = operation_node.operands
        start_date_sql = self.ast_node_to_query_term(start_date_operand).get_sql()
        end_date_sql = self.ast_node_to_query_term(end_date_operand).get_sql()

        date_value = date_part_operand.value
        spark_sql = SparkCalculationSqlBuilder._get_spark_date_diff_sql_str(date_value, start_date_sql, end_date_sql)
        return LiteralValue(spark_sql)


class CalculationSqlBuilderFactory:
    @classmethod
    def create_builder(
        cls,
        fdw: FeatureDefinitionWrapper,
        column_reference_resolver: ColumnReferenceResolver,
        compute_mode: ComputeMode,
    ) -> CalculationSqlBuilder:
        sql_builders = {
            ComputeMode.SPARK: SparkCalculationSqlBuilder,
            ComputeMode.RIFT: DuckDBCalculationSqlBuilder,
        }
        return sql_builders[compute_mode](fdw, column_reference_resolver)



from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.args import diff_options__client_pb2 as _diff_options__client_pb2
from tecton_proto.args import feature_service__client_pb2 as _feature_service__client_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from tecton_proto.common import schema__client_pb2 as _schema__client_pb2
from tecton_proto.common import spark_schema__client_pb2 as _spark_schema__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

CONTEXT_TYPE_MATERIALIZATION: ContextType
CONTEXT_TYPE_REALTIME: ContextType
CONTEXT_TYPE_UNSPECIFIED: ContextType
DESCRIPTOR: _descriptor.FileDescriptor
TIME_REFERENCE_MATERIALIZATION_END_TIME: TimeReference
TIME_REFERENCE_MATERIALIZATION_START_TIME: TimeReference
TIME_REFERENCE_UNBOUNDED_FUTURE: TimeReference
TIME_REFERENCE_UNBOUNDED_PAST: TimeReference
TIME_REFERENCE_UNSPECIFIED: TimeReference

class ConstantNode(_message.Message):
    __slots__ = ["bool_const", "float_const", "int_const", "null_const", "string_const"]
    BOOL_CONST_FIELD_NUMBER: _ClassVar[int]
    FLOAT_CONST_FIELD_NUMBER: _ClassVar[int]
    INT_CONST_FIELD_NUMBER: _ClassVar[int]
    NULL_CONST_FIELD_NUMBER: _ClassVar[int]
    STRING_CONST_FIELD_NUMBER: _ClassVar[int]
    bool_const: bool
    float_const: str
    int_const: str
    null_const: _empty_pb2.Empty
    string_const: str
    def __init__(self, string_const: _Optional[str] = ..., int_const: _Optional[str] = ..., float_const: _Optional[str] = ..., bool_const: bool = ..., null_const: _Optional[_Union[_empty_pb2.Empty, _Mapping]] = ...) -> None: ...

class ContextNode(_message.Message):
    __slots__ = ["context_type", "input_name"]
    CONTEXT_TYPE_FIELD_NUMBER: _ClassVar[int]
    INPUT_NAME_FIELD_NUMBER: _ClassVar[int]
    context_type: ContextType
    input_name: str
    def __init__(self, context_type: _Optional[_Union[ContextType, str]] = ..., input_name: _Optional[str] = ...) -> None: ...

class DataSourceNode(_message.Message):
    __slots__ = ["filter_end_time", "filter_start_time", "input_name", "schedule_offset", "start_time_offset", "virtual_data_source_id", "window", "window_unbounded", "window_unbounded_preceding"]
    FILTER_END_TIME_FIELD_NUMBER: _ClassVar[int]
    FILTER_START_TIME_FIELD_NUMBER: _ClassVar[int]
    INPUT_NAME_FIELD_NUMBER: _ClassVar[int]
    SCHEDULE_OFFSET_FIELD_NUMBER: _ClassVar[int]
    START_TIME_OFFSET_FIELD_NUMBER: _ClassVar[int]
    VIRTUAL_DATA_SOURCE_ID_FIELD_NUMBER: _ClassVar[int]
    WINDOW_FIELD_NUMBER: _ClassVar[int]
    WINDOW_UNBOUNDED_FIELD_NUMBER: _ClassVar[int]
    WINDOW_UNBOUNDED_PRECEDING_FIELD_NUMBER: _ClassVar[int]
    filter_end_time: FilterDateTime
    filter_start_time: FilterDateTime
    input_name: str
    schedule_offset: _duration_pb2.Duration
    start_time_offset: _duration_pb2.Duration
    virtual_data_source_id: _id__client_pb2.Id
    window: _duration_pb2.Duration
    window_unbounded: bool
    window_unbounded_preceding: bool
    def __init__(self, virtual_data_source_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., window: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., window_unbounded_preceding: bool = ..., window_unbounded: bool = ..., start_time_offset: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., schedule_offset: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., input_name: _Optional[str] = ..., filter_start_time: _Optional[_Union[FilterDateTime, _Mapping]] = ..., filter_end_time: _Optional[_Union[FilterDateTime, _Mapping]] = ...) -> None: ...

class FeatureViewNode(_message.Message):
    __slots__ = ["feature_reference", "feature_view_id", "input_name"]
    FEATURE_REFERENCE_FIELD_NUMBER: _ClassVar[int]
    FEATURE_VIEW_ID_FIELD_NUMBER: _ClassVar[int]
    INPUT_NAME_FIELD_NUMBER: _ClassVar[int]
    feature_reference: _feature_service__client_pb2.FeatureReference
    feature_view_id: _id__client_pb2.Id
    input_name: str
    def __init__(self, feature_view_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., feature_reference: _Optional[_Union[_feature_service__client_pb2.FeatureReference, _Mapping]] = ..., input_name: _Optional[str] = ...) -> None: ...

class FilterDateTime(_message.Message):
    __slots__ = ["relative_time", "timestamp"]
    RELATIVE_TIME_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    relative_time: RelativeTime
    timestamp: _timestamp_pb2.Timestamp
    def __init__(self, timestamp: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., relative_time: _Optional[_Union[RelativeTime, _Mapping]] = ...) -> None: ...

class Input(_message.Message):
    __slots__ = ["arg_index", "arg_name", "node"]
    ARG_INDEX_FIELD_NUMBER: _ClassVar[int]
    ARG_NAME_FIELD_NUMBER: _ClassVar[int]
    NODE_FIELD_NUMBER: _ClassVar[int]
    arg_index: int
    arg_name: str
    node: PipelineNode
    def __init__(self, arg_index: _Optional[int] = ..., arg_name: _Optional[str] = ..., node: _Optional[_Union[PipelineNode, _Mapping]] = ...) -> None: ...

class JoinInputsNode(_message.Message):
    __slots__ = ["nodes"]
    NODES_FIELD_NUMBER: _ClassVar[int]
    nodes: _containers.RepeatedCompositeFieldContainer[PipelineNode]
    def __init__(self, nodes: _Optional[_Iterable[_Union[PipelineNode, _Mapping]]] = ...) -> None: ...

class MaterializationContextNode(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class Pipeline(_message.Message):
    __slots__ = ["root"]
    ROOT_FIELD_NUMBER: _ClassVar[int]
    root: PipelineNode
    def __init__(self, root: _Optional[_Union[PipelineNode, _Mapping]] = ...) -> None: ...

class PipelineNode(_message.Message):
    __slots__ = ["constant_node", "context_node", "data_source_node", "feature_view_node", "join_inputs_node", "materialization_context_node", "request_data_source_node", "transformation_node"]
    CONSTANT_NODE_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_NODE_FIELD_NUMBER: _ClassVar[int]
    DATA_SOURCE_NODE_FIELD_NUMBER: _ClassVar[int]
    FEATURE_VIEW_NODE_FIELD_NUMBER: _ClassVar[int]
    JOIN_INPUTS_NODE_FIELD_NUMBER: _ClassVar[int]
    MATERIALIZATION_CONTEXT_NODE_FIELD_NUMBER: _ClassVar[int]
    REQUEST_DATA_SOURCE_NODE_FIELD_NUMBER: _ClassVar[int]
    TRANSFORMATION_NODE_FIELD_NUMBER: _ClassVar[int]
    constant_node: ConstantNode
    context_node: ContextNode
    data_source_node: DataSourceNode
    feature_view_node: FeatureViewNode
    join_inputs_node: JoinInputsNode
    materialization_context_node: MaterializationContextNode
    request_data_source_node: RequestDataSourceNode
    transformation_node: TransformationNode
    def __init__(self, transformation_node: _Optional[_Union[TransformationNode, _Mapping]] = ..., data_source_node: _Optional[_Union[DataSourceNode, _Mapping]] = ..., constant_node: _Optional[_Union[ConstantNode, _Mapping]] = ..., request_data_source_node: _Optional[_Union[RequestDataSourceNode, _Mapping]] = ..., feature_view_node: _Optional[_Union[FeatureViewNode, _Mapping]] = ..., materialization_context_node: _Optional[_Union[MaterializationContextNode, _Mapping]] = ..., context_node: _Optional[_Union[ContextNode, _Mapping]] = ..., join_inputs_node: _Optional[_Union[JoinInputsNode, _Mapping]] = ...) -> None: ...

class RelativeTime(_message.Message):
    __slots__ = ["offset", "time_reference"]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    TIME_REFERENCE_FIELD_NUMBER: _ClassVar[int]
    offset: _duration_pb2.Duration
    time_reference: TimeReference
    def __init__(self, time_reference: _Optional[_Union[TimeReference, str]] = ..., offset: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ...) -> None: ...

class RequestContext(_message.Message):
    __slots__ = ["schema", "tecton_schema"]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    TECTON_SCHEMA_FIELD_NUMBER: _ClassVar[int]
    schema: _spark_schema__client_pb2.SparkSchema
    tecton_schema: _schema__client_pb2.Schema
    def __init__(self, tecton_schema: _Optional[_Union[_schema__client_pb2.Schema, _Mapping]] = ..., schema: _Optional[_Union[_spark_schema__client_pb2.SparkSchema, _Mapping]] = ...) -> None: ...

class RequestDataSourceNode(_message.Message):
    __slots__ = ["input_name", "request_context"]
    INPUT_NAME_FIELD_NUMBER: _ClassVar[int]
    REQUEST_CONTEXT_FIELD_NUMBER: _ClassVar[int]
    input_name: str
    request_context: RequestContext
    def __init__(self, request_context: _Optional[_Union[RequestContext, _Mapping]] = ..., input_name: _Optional[str] = ...) -> None: ...

class TransformationNode(_message.Message):
    __slots__ = ["inputs", "transformation_id"]
    INPUTS_FIELD_NUMBER: _ClassVar[int]
    TRANSFORMATION_ID_FIELD_NUMBER: _ClassVar[int]
    inputs: _containers.RepeatedCompositeFieldContainer[Input]
    transformation_id: _id__client_pb2.Id
    def __init__(self, transformation_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., inputs: _Optional[_Iterable[_Union[Input, _Mapping]]] = ...) -> None: ...

class TimeReference(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class ContextType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class Secret(_message.Message):
    __slots__ = ["encrypted_value", "redacted_value", "value"]
    ENCRYPTED_VALUE_FIELD_NUMBER: _ClassVar[int]
    REDACTED_VALUE_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    encrypted_value: str
    redacted_value: str
    value: str
    def __init__(self, value: _Optional[str] = ..., redacted_value: _Optional[str] = ..., encrypted_value: _Optional[str] = ...) -> None: ...

class SecretReference(_message.Message):
    __slots__ = ["is_local", "key", "scope"]
    IS_LOCAL_FIELD_NUMBER: _ClassVar[int]
    KEY_FIELD_NUMBER: _ClassVar[int]
    SCOPE_FIELD_NUMBER: _ClassVar[int]
    is_local: bool
    key: str
    scope: str
    def __init__(self, scope: _Optional[str] = ..., key: _Optional[str] = ..., is_local: bool = ...) -> None: ...



import base64
import logging.config
import os
import sys
from urllib.parse import urlparse

from tecton_materialization.common.job_metadata import JobMetadataClient
from tecton_materialization.ray.job_status import JobStatusClient
from tecton_proto.materialization.job_metadata__client_pb2 import TectonManagedStage
from tecton_proto.materialization.params__client_pb2 import MaterializationTaskParams
from tecton_proto.materialization.params__client_pb2 import SecretMaterializationTaskParams
from tecton_proto.materialization.params__client_pb2 import SecretServiceParams


LOGGING_CONFIG = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "standard": {"format": "%(asctime)s [%(levelname)s] %(name)s: %(message)s"},
    },
    "handlers": {
        "default": {
            "level": "INFO",
            "formatter": "standard",
            "class": "logging.StreamHandler",
            "stream": sys.stderr,
        },
    },
    "loggers": {
        "": {  # root logger
            "handlers": ["default"],
            "level": "INFO",
        },
    },
}

logging.config.dictConfig(LOGGING_CONFIG)

logger = logging.getLogger(__name__)


def run_job(
    materialization_task_params: MaterializationTaskParams,
    secret_materialization_task_params: SecretMaterializationTaskParams,
):
    # nested import so that we can publish any import errors out to the job status table
    from tecton_materialization.ray.materialization_jobs import _ray
    from tecton_materialization.ray.materialization_jobs import run_materialization

    with _ray():
        run_materialization(materialization_task_params, secret_materialization_task_params)


def main():
    try:
        task_params = MaterializationTaskParams()
        secret_params = SecretMaterializationTaskParams()

        # Extract secret m13n params from env vars. VM-based Rift passes secrets via this env var.
        if "SECRET_MATERIALIZATION_PARAMS" in os.environ:
            logger.info("Found secret materialization params, parsing...")
            secret_params.ParseFromString(base64.standard_b64decode(os.environ["SECRET_MATERIALIZATION_PARAMS"]))

        # Extract standard m13n params from the object store
        if "MATERIALIZATION_TASK_PARAMS_S3_URL" in os.environ:
            parsed = urlparse(os.environ["MATERIALIZATION_TASK_PARAMS_S3_URL"])
            logger.info(f"Found materialization task params at {parsed}, parsing...")

            bucket_name = parsed.netloc
            object_key = parsed.path.lstrip("/")

            import boto3

            s3_client = boto3.client("s3")
            response = s3_client.get_object(Bucket=bucket_name, Key=object_key)
            content = response["Body"].read()

            task_params.ParseFromString(content)
        elif "MATERIALIZATION_TASK_PARAMS_GCS_URL" in os.environ:
            parsed = urlparse(os.environ["MATERIALIZATION_TASK_PARAMS_GCS_URL"])
            logger.info(f"Found materialization task params at {parsed}, parsing...")

            bucket_name = parsed.netloc
            object_key = parsed.path.lstrip("/")

            from google.cloud import storage

            storage_client = storage.Client()
            bucket = storage_client.bucket(bucket_name)
            blob = bucket.blob(object_key)
            content = blob.download_as_bytes()

            task_params.ParseFromString(content)
        # Legacy way of passing standard m13n params via env vars. Required for compatibility with Anyscale and GCP.
        # TODO: remove after we are completely off Anyscale
        elif "MATERIALIZATION_TASK_PARAMS" in os.environ:
            logger.info("Found legacy materialization task params in env vars, parsing...")
            task_params.ParseFromString(base64.standard_b64decode(os.environ["MATERIALIZATION_TASK_PARAMS"]))

            # Backwards compatibility for fetching secret access key from materialization task params,
            # if not already found in the secret env var.
            # Anyscale relies on this. Should be removed once Anyscale is completely gone
            # See https://tecton.atlassian.net/browse/TEC-20193
            if task_params.secrets_api_service_url and not secret_params.HasField("secret_service_params"):
                secret_service_params = SecretServiceParams()
                secret_service_params.secrets_api_service_url = task_params.secrets_api_service_url
                secret_service_params.secret_access_api_key = task_params.secret_access_api_key

                secret_params.secret_service_params.CopyFrom(secret_service_params)
        else:
            msg = "Materialization params were not provided"
            raise ValueError(msg)

        run_job(task_params, secret_params)
    except Exception:
        # make sure error details are published out before exiting
        try:
            job_status_client = JobStatusClient(JobMetadataClient.for_params(task_params))
            job_status_client.set_current_stage_failed(TectonManagedStage.ErrorType.UNEXPECTED_ERROR)
            job_status_client.set_overall_state(TectonManagedStage.State.ERROR)
        except Exception as e:
            # Error details won't appear in the job details UI
            logger.error(f"Failed to publish job error details to the job status table: {e}")
        raise


if __name__ == "__main__":
    main()



import inspect
from typing import Any
from typing import Dict
from typing import List
from typing import Mapping
from typing import Optional
from typing import Union

import pandas

from tecton_core import conf
from tecton_core import specs
from tecton_core.errors import UDF_ERROR
from tecton_core.errors import UDF_TYPE_ERROR
from tecton_core.errors import TectonInternalError
from tecton_core.errors import TectonValidationError
from tecton_core.id_helper import IdHelper
from tecton_core.pipeline.feature_pipeline import FeaturePipeline
from tecton_core.pipeline.feature_pipeline import NodeValueType
from tecton_core.pipeline.pipeline_common import CONSTANT_TYPE
from tecton_core.pipeline.pipeline_common import constant_node_to_value
from tecton_core.query.errors import UserCodeError
from tecton_core.realtime_context import REQUEST_TIMESTAMP_FIELD_NAME
from tecton_core.realtime_context import RealtimeContext
from tecton_core.resource_provider_context import ResourceProviderContext
from tecton_core.secret_management import SecretResolver
from tecton_proto.args.pipeline__client_pb2 import JoinInputsNode
from tecton_proto.args.pipeline__client_pb2 import Pipeline
from tecton_proto.args.pipeline__client_pb2 import PipelineNode
from tecton_proto.args.pipeline__client_pb2 import TransformationNode
from tecton_proto.args.transformation__client_pb2 import TransformationMode
from tecton_proto.common import secret__client_pb2 as secret_pb2
from tecton_proto.common.id__client_pb2 import Id as TectonId


NAMESPACE_SEPARATOR = "__"


class RealtimeFeaturePipeline(FeaturePipeline):
    def __init__(
        self,
        name: str,
        pipeline: Pipeline,
        transformations: List[specs.TransformationSpec],
        context_parameter_name: Optional[str] = None,
        events_df_timestamp_field: Optional[str] = None,
        pipeline_inputs: Optional[Dict[str, Union[Dict[str, Any], pandas.DataFrame, RealtimeContext]]] = None,
        secret_resolver: Optional[SecretResolver] = None,
        secret_references: Optional[Mapping[str, secret_pb2.SecretReference]] = None,
        resource_providers: Optional[Mapping[str, specs.ResourceProviderSpec]] = None,
    ) -> None:
        self._pipeline = pipeline
        self._name = name
        self._id_to_transformation = {t.id: t for t in transformations}
        self._events_df_timestamp_field = events_df_timestamp_field

        if self._pipeline.root.HasField("transformation_node"):
            root_transformation = self.get_transformation_by_id(
                self._pipeline.root.transformation_node.transformation_id
            )
            assert root_transformation.transformation_mode in (
                TransformationMode.TRANSFORMATION_MODE_PYTHON,
                TransformationMode.TRANSFORMATION_MODE_PANDAS,
            )
            # In Spark, the UDF cannot reference a proto enum, so instead save mode as a string
            self.mode = (
                "python"
                if root_transformation.transformation_mode == TransformationMode.TRANSFORMATION_MODE_PYTHON
                else "pandas"
            )
        elif self._pipeline.root.HasField("join_inputs_node"):
            self.mode = "no_transformation"
        else:
            msg = f"Unsupported pipeline root {self._pipeline.root}"
            raise ValueError(msg)

        # Access this conf value outside of the UDF to avoid doing it many times and avoiding any worker/driver state issues.
        self._should_check_output_schema = conf.get_bool("TECTON_PYTHON_ODFV_OUTPUT_SCHEMA_CHECK_ENABLED")
        if context_parameter_name and context_parameter_name in pipeline_inputs:
            # There is a validation that context must be provided, if it is in transformation body
            provided_context = pipeline_inputs[context_parameter_name]
            # Resolve Secrets
            if secret_references:
                provided_secrets = provided_context.secrets or {}

                invalid_provided_secrets = sorted(set(provided_secrets.keys()) - set(secret_references.keys()))
                if invalid_provided_secrets:
                    msg = f"Secrets: {invalid_provided_secrets} have been provided but are not declared in feature view definition."
                    raise TectonValidationError(msg)

                secrets_to_resolve = sorted(set(secret_references.keys()) - set(provided_secrets.keys()))

                if secrets_to_resolve is not None and secret_resolver is None:
                    msg = "Missing a secret resolver."
                    raise TectonInternalError(msg)
                resolved_secrets = secret_resolver.resolve_spec_map(
                    {k: v for k, v in secret_references.items() if k in secrets_to_resolve}
                )
                provided_context._secrets = {**provided_secrets, **resolved_secrets}

            # Resolve Resource Providers
            if resource_providers:
                provided_resources = provided_context.resources or {}
                required_resources = resource_providers or {}

                invalid_provided_resources = sorted(set(provided_resources.keys()) - set(required_resources.keys()))
                if invalid_provided_resources:
                    msg = f"Resource Providers: {invalid_provided_resources} have been provided but are not declared in feature view definition."
                    raise TectonValidationError(msg)

                def resolve_resource(resource_provider_spec):
                    if resource_provider_spec.secrets and secret_resolver is None:
                        msg = "Missing a secret resolver."
                        raise TectonInternalError(msg)
                    # TODO: Dedupe secrets resolving across resource providers.
                    signature_args = inspect.signature(resource_provider_spec.function).parameters.keys()
                    function_args = (
                        {
                            "context": ResourceProviderContext(
                                secrets=secret_resolver.resolve_spec_map(resource_provider_spec.secrets or {})
                            )
                        }
                        if "context" in signature_args
                        else {}
                    )
                    try:
                        return resource_provider_spec.function(**function_args)
                    except Exception as exc:
                        msg = f"Invoking Resource Provider: '{resource_provider_spec.name}' failed with exception."
                        raise UserCodeError(msg) from exc

                resources_to_resolve = sorted(set(required_resources.keys()) - set(provided_resources.keys()))
                resolved_resources = {
                    resource_key: resolve_resource(resource_provider_spec)
                    for resource_key, resource_provider_spec in resource_providers.items()
                    if resource_key in resources_to_resolve
                }
                provided_context._resources = {**provided_resources, **resolved_resources}

        self._pipeline_inputs = pipeline_inputs
        self._secret_resolver = secret_resolver
        self._secret_references = secret_references
        self._resource_providers = resource_providers

    def get_transformation_by_id(self, id: TectonId) -> specs.TransformationSpec:
        return self._id_to_transformation[IdHelper.to_string(id)]

    @property
    def is_pandas_mode(self):
        return self.mode == "pandas"

    @property
    def is_python_mode(self):
        return self.mode == "python"

    @property
    def is_no_transformation_mode(self):
        return self.mode == "no_transformation"

    @property
    def transformation_mode(self) -> TransformationMode:
        if self.mode == "python":
            return TransformationMode.TRANSFORMATION_MODE_PYTHON
        elif self.mode == "pandas":
            return TransformationMode.TRANSFORMATION_MODE_PANDAS
        elif self.mode == "no_transformation":
            return TransformationMode.TRANSFORMATION_MODE_NO_TRANSFORMATION
        else:
            msg = f"Unsupported mode in RealtimeFeaturePipeline, unable to determine transformation_mode. Mode: {self.mode}"
            raise ValueError(msg)

    def _context_node_to_value(self, pipeline_node: PipelineNode) -> Optional[Union[pandas.DataFrame, RealtimeContext]]:
        assert self._pipeline_inputs is not None
        return self._pipeline_inputs[pipeline_node.context_node.input_name]

    def _request_data_node_to_value(self, pipeline_node: PipelineNode) -> Union[Dict[str, Any], pandas.DataFrame]:
        assert self._pipeline_inputs is not None
        return self._pipeline_inputs[pipeline_node.request_data_source_node.input_name]

    def _feature_view_node_to_value(self, pipeline_node: PipelineNode) -> Union[Dict[str, Any], pandas.DataFrame]:
        assert self._pipeline_inputs is not None
        return self._pipeline_inputs[pipeline_node.feature_view_node.input_name]

    def _node_to_value(self, pipeline_node: PipelineNode) -> NodeValueType:
        if pipeline_node.HasField("constant_node"):
            return self._constant_node_to_value(pipeline_node.constant_node)
        elif pipeline_node.HasField("feature_view_node"):
            return self._feature_view_node_to_value(pipeline_node)
        elif pipeline_node.HasField("request_data_source_node"):
            return self._request_data_node_to_value(pipeline_node)
        elif pipeline_node.HasField("transformation_node"):
            return self._transformation_node_to_value(pipeline_node.transformation_node)
        elif pipeline_node.HasField("context_node"):
            return self._context_node_to_value(pipeline_node)
        elif pipeline_node.HasField("join_inputs_node"):
            return self._join_inputs_node_to_value(pipeline_node.join_inputs_node)
        elif pipeline_node.HasField("materialization_context_node"):
            msg = f"MaterializationContext is unsupported for {self._fco_name}s."
            raise ValueError(msg)
        else:
            msg = f"This is not yet implemented {pipeline_node}"
            raise NotImplementedError(msg)

    def _constant_node_to_value(self, pipeline_node: PipelineNode) -> CONSTANT_TYPE:
        return constant_node_to_value(pipeline_node)

    @staticmethod
    def _format_values_for_pandas_mode(input_df: pandas.DataFrame) -> pandas.DataFrame:
        for col in input_df.select_dtypes(include=["datetime64"]).columns:
            input_df[col] = pandas.to_datetime(input_df[col], utc=True)
        return input_df

    def _join_inputs_node_to_value(self, join_inputs_node: JoinInputsNode) -> Union[Dict[str, Any], pandas.DataFrame]:
        output = {}
        for input_node in join_inputs_node.nodes:
            prefix = _get_namespace_for_rtfv_source_node(input_node)
            node_value = self._node_to_value(input_node)
            if isinstance(node_value, RealtimeContext):
                output[f"{prefix}{REQUEST_TIMESTAMP_FIELD_NAME}"] = node_value.request_timestamp
                continue

            for key, value in node_value.items():
                output[f"{prefix}{key}"] = value
        return output

    def _transformation_node_to_value(
        self, transformation_node: TransformationNode
    ) -> Union[Dict[str, Any], pandas.DataFrame]:
        """Recursively translates inputs to values and then passes them to the transformation."""
        args: List[Union[str, int, float, bool]] = []
        kwargs = {}
        for transformation_input in transformation_node.inputs:
            input_node = transformation_input.node
            node_value = self._node_to_value(input_node)

            if transformation_input.HasField("arg_index"):
                assert len(args) == transformation_input.arg_index
                args.append(node_value)
            elif transformation_input.HasField("arg_name"):
                kwargs[transformation_input.arg_name] = node_value
            else:
                msg = f"Unknown argument type for Input node: {transformation_input}"
                raise KeyError(msg)

        return self._apply_transformation_function(transformation_node, args, kwargs)

    def _apply_transformation_function(
        self, transformation_node: TransformationNode, args: List[Any], kwargs: Dict[str, Any]
    ) -> Union[Dict[str, Any], pandas.DataFrame]:
        """For the given transformation node, returns the corresponding DataFrame transformation.

        If needed, resulted function is wrapped with a function that translates mode-specific input/output types to DataFrames.
        """
        transformation = self.get_transformation_by_id(transformation_node.transformation_id)
        mode = transformation.transformation_mode
        user_function = transformation.user_function

        if (
            mode != TransformationMode.TRANSFORMATION_MODE_PANDAS
            and mode != TransformationMode.TRANSFORMATION_MODE_PYTHON
        ):
            msg = f"Unsupported transformation mode({transformation.transformation_mode}) for {self._fco_name}s."
            raise KeyError(msg)

        try:
            resp = user_function(*args, **kwargs)
            return resp
        except TypeError as e:
            raise UDF_TYPE_ERROR(e)
        except Exception as e:
            raise UDF_ERROR(e, transformation.metadata.name)

    @property
    def _fco_name(self):
        return "Realtime Feature View"

    def run_with_inputs(
        self, inputs: Union[Dict[str, pandas.DataFrame], Dict[str, Any]]
    ) -> Union[CONSTANT_TYPE, Dict[str, Any], pandas.DataFrame, pandas.Series]:
        self._pipeline_inputs = inputs
        return self.get_dataframe()


def _get_namespace_for_rtfv_source_node(node: PipelineNode) -> str:
    if node.HasField("request_data_source_node"):
        return f"{node.request_data_source_node.input_name}{NAMESPACE_SEPARATOR}"
    elif node.HasField("feature_view_node"):
        return f"{node.feature_view_node.feature_reference.namespace}{NAMESPACE_SEPARATOR}"
    elif node.HasField("context_node"):
        return f"{node.context_node.input_name}{NAMESPACE_SEPARATOR}"
    return ""



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class Pair(_message.Message):
    __slots__ = ["first", "second"]
    FIRST_FIELD_NUMBER: _ClassVar[int]
    SECOND_FIELD_NUMBER: _ClassVar[int]
    first: str
    second: str
    def __init__(self, first: _Optional[str] = ..., second: _Optional[str] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/pipeline__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.args import diff_options__client_pb2 as tecton__proto_dot_args_dot_diff__options____client__pb2
from tecton_proto.args import feature_service__client_pb2 as tecton__proto_dot_args_dot_feature__service____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import schema__client_pb2 as tecton__proto_dot_common_dot_schema____client__pb2
from tecton_proto.common import spark_schema__client_pb2 as tecton__proto_dot_common_dot_spark__schema____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(tecton_proto/args/pipeline__client.proto\x12\x11tecton_proto.args\x1a\x1egoogle/protobuf/duration.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,tecton_proto/args/diff_options__client.proto\x1a/tecton_proto/args/feature_service__client.proto\x1a$tecton_proto/common/id__client.proto\x1a(tecton_proto/common/schema__client.proto\x1a.tecton_proto/common/spark_schema__client.proto\"K\n\x08Pipeline\x12\x33\n\x04root\x18\x01 \x01(\x0b\x32\x1f.tecton_proto.args.PipelineNodeR\x04rootJ\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04\"\xd0\x05\n\x0cPipelineNode\x12X\n\x13transformation_node\x18\x01 \x01(\x0b\x32%.tecton_proto.args.TransformationNodeH\x00R\x12transformationNode\x12M\n\x10\x64\x61ta_source_node\x18\x02 \x01(\x0b\x32!.tecton_proto.args.DataSourceNodeH\x00R\x0e\x64\x61taSourceNode\x12\x46\n\rconstant_node\x18\x03 \x01(\x0b\x32\x1f.tecton_proto.args.ConstantNodeH\x00R\x0c\x63onstantNode\x12\x63\n\x18request_data_source_node\x18\x04 \x01(\x0b\x32(.tecton_proto.args.RequestDataSourceNodeH\x00R\x15requestDataSourceNode\x12P\n\x11\x66\x65\x61ture_view_node\x18\x05 \x01(\x0b\x32\".tecton_proto.args.FeatureViewNodeH\x00R\x0f\x66\x65\x61tureViewNode\x12q\n\x1cmaterialization_context_node\x18\x07 \x01(\x0b\x32-.tecton_proto.args.MaterializationContextNodeH\x00R\x1amaterializationContextNode\x12\x43\n\x0c\x63ontext_node\x18\x08 \x01(\x0b\x32\x1e.tecton_proto.args.ContextNodeH\x00R\x0b\x63ontextNode\x12M\n\x10join_inputs_node\x18\t \x01(\x0b\x32!.tecton_proto.args.JoinInputsNodeH\x00R\x0ejoinInputsNodeB\x0b\n\tnode_typeJ\x04\x08\x06\x10\x07\"\x89\x01\n\x15RequestDataSourceNode\x12Q\n\x0frequest_context\x18\x01 \x01(\x0b\x32!.tecton_proto.args.RequestContextB\x05\x92M\x02 \x04R\x0erequestContext\x12\x1d\n\ninput_name\x18\x02 \x01(\tR\tinputName\"\x8c\x01\n\x0eRequestContext\x12@\n\rtecton_schema\x18\x02 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\x0ctectonSchema\x12\x38\n\x06schema\x18\x01 \x01(\x0b\x32 .tecton_proto.common.SparkSchemaR\x06schema\"\xc3\x01\n\x0f\x46\x65\x61tureViewNode\x12?\n\x0f\x66\x65\x61ture_view_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12P\n\x11\x66\x65\x61ture_reference\x18\x02 \x01(\x0b\x32#.tecton_proto.args.FeatureReferenceR\x10\x66\x65\x61tureReference\x12\x1d\n\ninput_name\x18\x03 \x01(\tR\tinputName\"\x8c\x01\n\x12TransformationNode\x12\x44\n\x11transformation_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10transformationId\x12\x30\n\x06inputs\x18\x02 \x03(\x0b\x32\x18.tecton_proto.args.InputR\x06inputs\"G\n\x0eJoinInputsNode\x12\x35\n\x05nodes\x18\x02 \x03(\x0b\x32\x1f.tecton_proto.args.PipelineNodeR\x05nodes\"\x84\x01\n\x05Input\x12\x1d\n\targ_index\x18\x01 \x01(\x05H\x00R\x08\x61rgIndex\x12\x1b\n\x08\x61rg_name\x18\x02 \x01(\tH\x00R\x07\x61rgName\x12\x33\n\x04node\x18\x03 \x01(\x0b\x32\x1f.tecton_proto.args.PipelineNodeR\x04nodeB\n\n\x08\x61rg_type\"\x81\x05\n\x0e\x44\x61taSourceNode\x12L\n\x16virtual_data_source_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x13virtualDataSourceId\x12\x33\n\x06window\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationH\x00R\x06window\x12\x45\n\x1awindow_unbounded_preceding\x18\x04 \x01(\x08\x42\x05\x92M\x02 \x12H\x00R\x18windowUnboundedPreceding\x12\x32\n\x10window_unbounded\x18\x06 \x01(\x08\x42\x05\x92M\x02 \x12H\x00R\x0fwindowUnbounded\x12N\n\x11start_time_offset\x18\x07 \x01(\x0b\x32\x19.google.protobuf.DurationB\x05\x92M\x02 \x12H\x00R\x0fstartTimeOffset\x12\x42\n\x0fschedule_offset\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0escheduleOffset\x12\x1d\n\ninput_name\x18\x05 \x01(\tR\tinputName\x12T\n\x11\x66ilter_start_time\x18\x08 \x01(\x0b\x32!.tecton_proto.args.FilterDateTimeB\x05\x92M\x02 \x12R\x0f\x66ilterStartTime\x12P\n\x0f\x66ilter_end_time\x18\t \x01(\x0b\x32!.tecton_proto.args.FilterDateTimeB\x05\x92M\x02 \x12R\rfilterEndTimeB\x16\n\x14\x64\x61ta_window_override\"\xaa\x01\n\x0e\x46ilterDateTime\x12:\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00R\ttimestamp\x12\x46\n\rrelative_time\x18\x02 \x01(\x0b\x32\x1f.tecton_proto.args.RelativeTimeH\x00R\x0crelativeTimeB\x14\n\x12time_specification\"\x8a\x01\n\x0cRelativeTime\x12G\n\x0etime_reference\x18\x01 \x01(\x0e\x32 .tecton_proto.args.TimeReferenceR\rtimeReference\x12\x31\n\x06offset\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\x06offset\"\xe0\x01\n\x0c\x43onstantNode\x12#\n\x0cstring_const\x18\x01 \x01(\tH\x00R\x0bstringConst\x12\x1d\n\tint_const\x18\x02 \x01(\tH\x00R\x08intConst\x12!\n\x0b\x66loat_const\x18\x03 \x01(\tH\x00R\nfloatConst\x12\x1f\n\nbool_const\x18\x04 \x01(\x08H\x00R\tboolConst\x12\x37\n\nnull_const\x18\x05 \x01(\x0b\x32\x16.google.protobuf.EmptyH\x00R\tnullConstB\x0f\n\rconstant_type\"\x1c\n\x1aMaterializationContextNode\"o\n\x0b\x43ontextNode\x12\x41\n\x0c\x63ontext_type\x18\x01 \x01(\x0e\x32\x1e.tecton_proto.args.ContextTypeR\x0b\x63ontextType\x12\x1d\n\ninput_name\x18\x02 \x01(\tR\tinputName*\xd3\x01\n\rTimeReference\x12\x1e\n\x1aTIME_REFERENCE_UNSPECIFIED\x10\x00\x12-\n)TIME_REFERENCE_MATERIALIZATION_START_TIME\x10\x01\x12+\n\'TIME_REFERENCE_MATERIALIZATION_END_TIME\x10\x02\x12!\n\x1dTIME_REFERENCE_UNBOUNDED_PAST\x10\x03\x12#\n\x1fTIME_REFERENCE_UNBOUNDED_FUTURE\x10\x04*h\n\x0b\x43ontextType\x12\x1c\n\x18\x43ONTEXT_TYPE_UNSPECIFIED\x10\x00\x12 \n\x1c\x43ONTEXT_TYPE_MATERIALIZATION\x10\x01\x12\x19\n\x15\x43ONTEXT_TYPE_REALTIME\x10\x02\x42;\n\x0f\x63om.tecton.argsP\x01Z&github.com/tecton-ai/tecton_proto/args')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.pipeline__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001Z&github.com/tecton-ai/tecton_proto/args'
  _REQUESTDATASOURCENODE.fields_by_name['request_context']._options = None
  _REQUESTDATASOURCENODE.fields_by_name['request_context']._serialized_options = b'\222M\002 \004'
  _DATASOURCENODE.fields_by_name['window_unbounded_preceding']._options = None
  _DATASOURCENODE.fields_by_name['window_unbounded_preceding']._serialized_options = b'\222M\002 \022'
  _DATASOURCENODE.fields_by_name['window_unbounded']._options = None
  _DATASOURCENODE.fields_by_name['window_unbounded']._serialized_options = b'\222M\002 \022'
  _DATASOURCENODE.fields_by_name['start_time_offset']._options = None
  _DATASOURCENODE.fields_by_name['start_time_offset']._serialized_options = b'\222M\002 \022'
  _DATASOURCENODE.fields_by_name['filter_start_time']._options = None
  _DATASOURCENODE.fields_by_name['filter_start_time']._serialized_options = b'\222M\002 \022'
  _DATASOURCENODE.fields_by_name['filter_end_time']._options = None
  _DATASOURCENODE.fields_by_name['filter_end_time']._serialized_options = b'\222M\002 \022'
  _TIMEREFERENCE._serialized_start=3341
  _TIMEREFERENCE._serialized_end=3552
  _CONTEXTTYPE._serialized_start=3554
  _CONTEXTTYPE._serialized_end=3658
  _PIPELINE._serialized_start=380
  _PIPELINE._serialized_end=455
  _PIPELINENODE._serialized_start=458
  _PIPELINENODE._serialized_end=1178
  _REQUESTDATASOURCENODE._serialized_start=1181
  _REQUESTDATASOURCENODE._serialized_end=1318
  _REQUESTCONTEXT._serialized_start=1321
  _REQUESTCONTEXT._serialized_end=1461
  _FEATUREVIEWNODE._serialized_start=1464
  _FEATUREVIEWNODE._serialized_end=1659
  _TRANSFORMATIONNODE._serialized_start=1662
  _TRANSFORMATIONNODE._serialized_end=1802
  _JOININPUTSNODE._serialized_start=1804
  _JOININPUTSNODE._serialized_end=1875
  _INPUT._serialized_start=1878
  _INPUT._serialized_end=2010
  _DATASOURCENODE._serialized_start=2013
  _DATASOURCENODE._serialized_end=2654
  _FILTERDATETIME._serialized_start=2657
  _FILTERDATETIME._serialized_end=2827
  _RELATIVETIME._serialized_start=2830
  _RELATIVETIME._serialized_end=2968
  _CONSTANTNODE._serialized_start=2971
  _CONSTANTNODE._serialized_end=3195
  _MATERIALIZATIONCONTEXTNODE._serialized_start=3197
  _MATERIALIZATIONCONTEXTNODE._serialized_end=3225
  _CONTEXTNODE._serialized_start=3227
  _CONTEXTNODE._serialized_end=3338
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/server_groups/server_group_config__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.realtime import instance_group__client_pb2 as tecton__proto_dot_realtime_dot_instance__group____client__pb2
from tecton_proto.server_groups import server_group_states__client_pb2 as tecton__proto_dot_server__groups_dot_server__group__states____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n<tecton_proto/server_groups/server_group_config__client.proto\x12\x1atecton_proto.server_groups\x1a\x32tecton_proto/realtime/instance_group__client.proto\x1a<tecton_proto/server_groups/server_group_states__client.proto\"\x86\x01\n\x18ServerGroupClusterConfig\x12j\n\x16transform_server_group\x18\x01 \x03(\x0b\x32\x34.tecton_proto.server_groups.TransformServerGroupInfoR\x14transformServerGroup\"\xfa\x03\n\x18TransformServerGroupInfo\x12&\n\x0fserver_group_id\x18\x01 \x01(\tR\rserverGroupId\x12\\\n\x12\x61utoscaling_policy\x18\x02 \x01(\x0b\x32-.tecton_proto.server_groups.AutoscalingPolicyR\x11\x61utoscalingPolicy\x12p\n\x19\x61ws_instance_group_config\x18\x03 \x01(\x0b\x32\x33.tecton_proto.realtime.AWSInstanceGroupUpdateConfigH\x00R\x16\x61wsInstanceGroupConfig\x12\x83\x01\n\x15\x65nvironment_variables\x18\x04 \x03(\x0b\x32N.tecton_proto.server_groups.TransformServerGroupInfo.EnvironmentVariablesEntryR\x14\x65nvironmentVariables\x1aG\n\x19\x45nvironmentVariablesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x17\n\x15instance_group_configB\x1b\n\x17\x63om.tecton.servergroupsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.server_groups.server_group_config__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027com.tecton.servergroupsP\001'
  _TRANSFORMSERVERGROUPINFO_ENVIRONMENTVARIABLESENTRY._options = None
  _TRANSFORMSERVERGROUPINFO_ENVIRONMENTVARIABLESENTRY._serialized_options = b'8\001'
  _SERVERGROUPCLUSTERCONFIG._serialized_start=207
  _SERVERGROUPCLUSTERCONFIG._serialized_end=341
  _TRANSFORMSERVERGROUPINFO._serialized_start=344
  _TRANSFORMSERVERGROUPINFO._serialized_end=850
  _TRANSFORMSERVERGROUPINFO_ENVIRONMENTVARIABLESENTRY._serialized_start=754
  _TRANSFORMSERVERGROUPINFO_ENVIRONMENTVARIABLESENTRY._serialized_end=825
# @@protoc_insertion_point(module_scope)



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class ContainerImage(_message.Message):
    __slots__ = ["image_digest", "image_tag", "image_uri", "repository_name"]
    IMAGE_DIGEST_FIELD_NUMBER: _ClassVar[int]
    IMAGE_TAG_FIELD_NUMBER: _ClassVar[int]
    IMAGE_URI_FIELD_NUMBER: _ClassVar[int]
    REPOSITORY_NAME_FIELD_NUMBER: _ClassVar[int]
    image_digest: str
    image_tag: str
    image_uri: str
    repository_name: str
    def __init__(self, repository_name: _Optional[str] = ..., image_uri: _Optional[str] = ..., image_digest: _Optional[str] = ..., image_tag: _Optional[str] = ...) -> None: ...



from tecton_proto.common import container_image__client_pb2 as _container_image__client_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from tecton_proto.common import scaling_config__client_pb2 as _scaling_config__client_pb2
from tecton_proto.common import server_group_type__client_pb2 as _server_group_type__client_pb2
from tecton_proto.data import fco_metadata__client_pb2 as _fco_metadata__client_pb2
from tecton_proto.validation import validator__client_pb2 as _validator__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class FeatureServerGroup(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class ServerGroup(_message.Message):
    __slots__ = ["autoscaling_config", "fco_metadata", "feature_server_group", "options", "provisioned_scaling_config", "server_group_id", "transform_server_group", "type", "validation_args"]
    class OptionsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    AUTOSCALING_CONFIG_FIELD_NUMBER: _ClassVar[int]
    FCO_METADATA_FIELD_NUMBER: _ClassVar[int]
    FEATURE_SERVER_GROUP_FIELD_NUMBER: _ClassVar[int]
    OPTIONS_FIELD_NUMBER: _ClassVar[int]
    PROVISIONED_SCALING_CONFIG_FIELD_NUMBER: _ClassVar[int]
    SERVER_GROUP_ID_FIELD_NUMBER: _ClassVar[int]
    TRANSFORM_SERVER_GROUP_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    VALIDATION_ARGS_FIELD_NUMBER: _ClassVar[int]
    autoscaling_config: _scaling_config__client_pb2.AutoscalingConfig
    fco_metadata: _fco_metadata__client_pb2.FcoMetadata
    feature_server_group: FeatureServerGroup
    options: _containers.ScalarMap[str, str]
    provisioned_scaling_config: _scaling_config__client_pb2.ProvisionedScalingConfig
    server_group_id: _id__client_pb2.Id
    transform_server_group: TransformServerGroup
    type: _server_group_type__client_pb2.ServerGroupType
    validation_args: _validator__client_pb2.ServerGroupValidationArgs
    def __init__(self, server_group_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., fco_metadata: _Optional[_Union[_fco_metadata__client_pb2.FcoMetadata, _Mapping]] = ..., type: _Optional[_Union[_server_group_type__client_pb2.ServerGroupType, str]] = ..., transform_server_group: _Optional[_Union[TransformServerGroup, _Mapping]] = ..., feature_server_group: _Optional[_Union[FeatureServerGroup, _Mapping]] = ..., options: _Optional[_Mapping[str, str]] = ..., validation_args: _Optional[_Union[_validator__client_pb2.ServerGroupValidationArgs, _Mapping]] = ..., autoscaling_config: _Optional[_Union[_scaling_config__client_pb2.AutoscalingConfig, _Mapping]] = ..., provisioned_scaling_config: _Optional[_Union[_scaling_config__client_pb2.ProvisionedScalingConfig, _Mapping]] = ...) -> None: ...

class TransformServerGroup(_message.Message):
    __slots__ = ["environment_id", "environment_name", "image_info"]
    ENVIRONMENT_ID_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_NAME_FIELD_NUMBER: _ClassVar[int]
    IMAGE_INFO_FIELD_NUMBER: _ClassVar[int]
    environment_id: str
    environment_name: str
    image_info: _container_image__client_pb2.ContainerImage
    def __init__(self, environment_id: _Optional[str] = ..., environment_name: _Optional[str] = ..., image_info: _Optional[_Union[_container_image__client_pb2.ContainerImage, _Mapping]] = ...) -> None: ...
